#include <Cells.h>

Cells ::  Cells (const String& cellmodeltype, MenuSystem& menu, int level = MAIN) 
{ 
  cellmodel = createODE (cellmodeltype);
  cellmodel->define (menu, level+1); 
  
  noStates = cellmodel->evalSize();
  cpu_time_forward = DUMMY;
  total_forward_time = 0;
  noAux = 8;
  V_index = cellmodel->getTransmembraneIndex();
  cout << "V_index: " << cellmodel->getTransmembraneIndex() << endl;  
}

void Cells :: attachDiff(Mono1MG* diff) { 
  diffusion = diff;
}

void Cells :: destruct() {
//    DbP(""); grid.detach(); 
//    DbP(""); current.detach(); 
//    DbP(""); u.detach(); 
//    DbP(""); aux.detach(); 
  } 

void Cells :: defineStatic (MenuSystem& menu, int  level = MAIN ) {
  
  String description = "Parameters for the ODE's";
  String command = "Cells";
  MenuSystem::makeSubMenuHeader (menu, description, command, level, 'P');
  menu.setCommandPrefix("Cells");
  menu.addItem (level,        // menu level (level+1 indicates a submenu)
                "ICfile",   // menu item command/name
                "ICfile",   
                "file containing the intial contions at points",
	        "2Dtorso.ICstim",
                "S");         // valid answer: string

  menu.unsetCommandPrefix();
}
    
void Cells :: define (MenuSystem& menu, int level = MAIN) {
  defineStatic (menu,level);
}

void  Cells :: scan   (MenuSystem& menu)   { 
  cellmodel->scan(menu); 
  menu.setCommandPrefix("Cells");
  ICfile = menu.get ("ICfile");
  menu.unsetCommandPrefix();
}

void Cells :: dump () {
  /*   
  Ptv(real) ui;

  for (int i = 1; i <= noStates; i++) {
    Os u_file (oform("state%d.track",i), APPEND);
    for (int n = 1; n <= userdef->getTrackNodes().size(); n++) {	
      u->valueNode(ui, userdef->getTrackNodes()(n));
      u_file << ui(i) << " ";
    }
    u_file << endl;
  }
  */
}

void Cells :: attachGridAndTimer(GridFE& globalgrid, TimePrm& globaltip)
{   	
  DPTRACE("Cells::attachGridAndTimer");
  grid.rebind(globalgrid);
  u.rebind(new FieldsFE(grid(),cellmodel->evalSize(),"states"));
  u->fill(cellmodel->currentState());
  DBP("ok");
  aux.rebind(new FieldsFE(grid(),noAux,"aux"));  
  Ptv(real) aux0(noAux); aux0(1) = -999.9; aux0(2) = -100.0; 
  aux0(3) = 0.0; aux0(4) = 0.0; aux0(5) = -999.9;
  aux0(6) = 0.0; aux0(7) = 0.0; aux0(8) = 0.0;
  aux->fill(aux0);  
  readICFile(ICfile);

  Ptv(real) ui;
  //u->valueNode(ui,1);	// reads the value of grid node i of u into ui

  tip.rebind(globaltip);
  noNodes = grid->getNoNodes();
  cellmodel->attachTimer(tip());
}


void Cells:: cputime ()
{
  // assume that cpu.initTime() was correctly called such that cpu.getTime()
  // now gives the correct cpu time:

   cpu_time_forward = cpu.getInterval();
   total_forward_time += cpu_time_forward;
   Os logfile (logfile_name, APPEND);
   logfile << "\nXTR: forward=" << cpu_time_forward;

}

bool Cells:: ok () const
{
  DPTRACE("Cells::ok");
  bool b = true;
  HANDLE_OK("Cells",u,b);        
  HANDLE_OK("Cells",grid,b);
 return b;
}


void Cells:: resultReport ()
{

  String logfile_name2 = logfile_name;
  logfile_name2 += '2';
  Os logfile2 (logfile_name2, APPEND);

  //int NM = userdef->getNoSteps()*userdef->getMuscleGrid().getNoNodes();
  //real avg_forward_time = 1000*total_forward_time/NM;

  /*
  logfile2 << "Forward time " << total_forward_time << " Avg: " << avg_forward_time <<endl;

  String logfile_name3 = logfile_name;
  logfile_name3 += '3';
  Os logfile3 (logfile_name3, APPEND);
  logfile3 << total_forward_time  << " ";
  */
  /*
  if (userdef->getPlotLevel()>0) 
    database->dump (aux()(1),tip.getPtr(),"time");
  */
}

void Cells :: forward(real T0, real dt)
{

  cpu.initTime();

  Ptv(real) ui(noStates);
  Ptv(real) auxi(noAux);

  //grid.rebind(userdef->getMuscleGrid());	// 32
  noNodes = grid->getNoNodes();
  //u.rebind(userdef->getStates());
  //aux.rebind(userdef->getAux());
  int cnt = 0;

  int i,j;
  for (i=1; i<=noNodes; i++) {
    u->valueNode(ui,i);

    ui(V_index) = diffusion->valueNode(i);

    //    if (ui(V_index)<-120) ui(V_index)=-120; // for now
    if (ui(V_index)>-20) cnt++;
    aux->valueNode(auxi,i);
    auxi(4) = T0 - dt;
    cellmodel->setParameters(auxi);
    //if (i==1){cout << "verdi fr forward:"; ui.print(cout);}
    cellmodel->forward(ui,T0,dt,auxi);	// integrates one timestep
    //if (i==1){cout << "\n verdi etter forward:"; ui.print(cout);}
    for (j = 1; j <= noStates; j++)
      u()(j).valueNode(i) = ui(j);	// updates the fields

    diffusion->valueNode(i) = ui(V_index); //update the v_field according to new splitting
    diffusion->v_prev->valueNode(i) = ui(V_index); //new v is initial condition for PDE step

    for (j = 1; j <= noAux; j++)
      aux()(j).valueNode(i) = auxi(j);	// updates the fields
  }

  Os logfile (logfile_name, APPEND);
  logfile  << cnt << " out of " << noNodes << " active " << endl;

  cputime();
}



void  Cells :: readICFile(Is is) {

  const int nsd = grid->getNoSpaceDim();
  int i, j, k, no_items;
  Ptv(real) coor;
  Mat(real) ICdata;
  real dist;
 
  is >> no_items;  

  cout << no_items << endl;  

  ICdata.redim(no_items,nsd+5);
  ICdata.print(cout);


  coor.redim(nsd);
  for (i = 1; i <= no_items; i++) {
    is >> ICdata(i,1); // type
    for (j = 2; j <= nsd+1; j++) 
      is >> ICdata(i,j);
    is >> ICdata(i,nsd+2); // radius
    is >> ICdata(i,nsd+3); // value
    is >> ICdata(i,nsd+4); // start time
    is >> ICdata(i,nsd+5); // stop time
  }
  cout << "nsd in readIC: " << nsd << endl;
  ICdata.print(cout);




//  if  (userdef->getPlotLevel()>0) {
  //    Os gfile ("initialpoints.mtv", NEWFILE);
  //  gfile << "$ DATA=CURVE2D\n\n  linetype 0 markertype=1\n";
//   }



  for (k = 1; k <= no_items; k++) 

  for (i = 1; i <= grid->getNoNodes(); i++)	{
    coor = aux()(1).getPt(i);

    if (ICdata(k,1) == 0) { //sphere
      dist = 0;
      for (j = 1; j <= nsd; j++) dist += (coor(j)-ICdata(k,j+1))*(coor(j)-ICdata(k,j+1)); 
      //    cout << "dist: " << dist << " radius: " << ICdata(k,nsd+2) << endl;
      if (sqrt(dist) < ICdata(k,nsd+2))  {
	aux()(6).valueNode(i) = ICdata(k,nsd+3);
	aux()(7).valueNode(i) = ICdata(k,nsd+4);
	aux()(8).valueNode(i) = ICdata(k,nsd+5);
	//if  (userdef->getPlotLevel()>0) 
	//	gfile << coor << endl;
	//cout << oform("setting node %d to %5.2f\n",i,ICdata(k,nsd+3));
      }
    } 
    else { //plane
      
      if (coor( int (ICdata(k,1)) ) < ICdata(k,nsd+2))  {
	aux()(6).valueNode(i) = ICdata(k,nsd+3);
	aux()(7).valueNode(i) = ICdata(k,nsd+4);
	aux()(8).valueNode(i) = ICdata(k,nsd+5);
	//cout << oform("setting node %d to %5.2f\n",i,ICdata(k,nsd+3));
      }

    }
  }

// if  (userdef->getPlotLevel()>0)  
  //gfile << "$END\n";


}












