                                                               // -*- C++ -*-
// $Id: Cells.h,v 1.1.1.1 2003/11/25 16:35:31 glennli Exp $
// prevent multiple inclusion of Cells.h
#ifndef Cells_h_IS_INCLUDED
#define Cells_h_IS_INCLUDED

#include <FEM.h>           // FEM algorithms, FieldFE, GridFE etc
#include <SimCase.h>       // menu system utilities
#include <SaveSimRes.h>    // storage tool for later visualization
#include <CPUclock.h> 
#include <CellModel.h>
#include <create.h>
#include <Mono1MG.h>         

#include <VecSimplest_VecSimple_real.h>


class Mono1MG;

// ------------------------- Cells -----------------------

/*<Cells:*/

class Cells : public SimCase
{


protected:
  // general data:
  //Handle(SaveSimRes) database;
  Handle(TimePrm)   tip;   

  Handle(GridFE)    grid;   // finite element grid
  Handle(FieldsFE)  u;     // cellular states 
  Handle(FieldsFE)  aux;     // auxilary data, e.g. activation time

  //Handle(FieldFE)  current;     // holds ion current field
  //Handle(FieldFE) current_fac; //holds coefficient for linear part of current
  // update ion after each forward(), see ionicCurrent for how
  // rewrite the body of ionicCurrent, same header.

                           // holds the state varibles in each node
  CPUclock      cpu;
  double        cpu_time_forward;
  void cputime();

  CellModel*	cellmodel;  // Class holding the description of the
                            // chemical reaction in each cell.
  String 	ICfile; 
  Mono1MG* 	diffusion;   // Needed to obtain the transmembrane potential
  int noStates, noNodes;

  real total_forward_time;
  int noAux;
  int V_index; // the index of the transmembrane potential in the state vector

public:
  Cells (const String& cellmodeltype, MenuSystem& menu, int level);
 ~Cells () {}

  void destruct();
  void dump (); 
  void attachGridAndTimer(GridFE& globalgrid, TimePrm& globaltip);
  real getTotalForwardTime() {return total_forward_time;}

  void attachDiff(Mono1MG* diffusion);

  static void defineStatic (MenuSystem& menu, int  level = MAIN );
  void define (MenuSystem& menu, int level = MAIN);
  void scan   (MenuSystem& menu); //virtual
  
  void readICFile(Is is);
  
  void forward(real T0, real dt); 

  bool ok () const;             // is the simulator properly initialized?

  void resultReport ();  //virtual?

  FieldsFE& getStates() {return u();}
  FieldsFE& getAux() {return aux();}
};

/*>Cells:*/

/*Class:Cells

NAME: 	Cells - Holds and integrates the state varibales the
        ionic model.

SYNTAX:	           @Cells

KEYWORDS:          

   Cells, Heart


DESCRIPTION: 
   
   The class implements a data structure and methods for
   holding and updating the state variables of the 
   ionic model in each node


CONSTRUCTORS AND INITIALIZATION:

   The constructor takes a string parameter describing the
   cell model type as input. Also a pointer to the menu 
   system is sendt as input. In this way the correct
   menu can be defined based on the cell model type.


MEMBER FUNCTIONS:

"forward" - loops over all the nodes in the grid and integrate
one timestep forward in time

"ionicCurrent" - returns the ionic current in a particular
evaluation point in a given finite element. 

"scan" - calls the 'scan' of CellModel


EXAMPLE:           


SEE ALSO:          

   Heart, CellModel

DEVELOPED BY:   

AUTHOR:	        

   Glenn Terje Lines

End:
*/
#endif
/*
$Log: Cells.h,v $
Revision 1.1.1.1  2003/11/25 16:35:31  glennli
INF-5610 oblig

*/

