#ifndef Mono1MG_h_IS_INCLUDED   // avoid multiple inclusions of Mono1MG.h
#define Mono1MG_h_IS_INCLUDED

#include <FEM.h>            // FEM algorithms, FieldFE, GridFE etc
#include <DegFreeFE.h>      // mapping: nodal values -> unknowns in linear sys.
#include <LinEqAdmFE.h>     // linear systems: storage and solution
#include <TimePrm.h>        // time loop parameters
#include <SaveSimRes.h>     // for storage of fields and curves
#include <FieldSummary.h>   // for summary of scalar field history
#include <MGtools.h>

#include <Cells.h>

class Cells;

class Mono1MG : public FEM
{
public:
  Vec(real)          linsol;     // solution of linear system
  int                no_of_grids;
  Handle(MGtools)    mgtools;
  Handle(GridFE)     grid;     
  Handle(DegFreeFE)  dof;  

  VecSimplest(int) tracknodes;


  real beta;                             //surface to volume ratio
  real theta;                            //time disc parameter

  Handle(FieldFE)    v;          // solution u^q
  Handle(FieldFE)    v_prev;     // u^{q-1} (previous time level)
  FieldSummary       v_summary;  // time history of extremes of u
  Handle(LinEqAdmFE) lineq;      // linear system and solvers
  Handle(TimePrm)    tip;        // time discretization parameters

  Handle(FieldFE)    error;
  NumItgPoints       error_itg_pt_tp;
  real               L1_error, L2_error, Linf_error;
  Handle(SaveSimRes) database;    

  Handle(FieldsFE)   flux;       // flux = smooth -k*grad(u)
  real               diffusion_coeff;  // constant k
  Handle(FieldFunc)  vanal;      // functor for analytical solution

  String cellmodeltype;
  Cells* cells;                  //calculates the ionic current term

  // in case of surface integrals or debugging:
  virtual void calcElmMatVec 
     (int e, ElmMatVec& elmat, FiniteElement& fe);

  bool compute_error_norms;
  virtual void computeErrorNorms();
  virtual void createTrackPoints();
  virtual void dumpTrackValues();


  //virtual void initCondFields();
  //virtual void readICfile(Is is);
                        
  virtual void integrands (ElmMatVec& elmat, const FiniteElement& fe);
  virtual void timeLoop();
  virtual void solveAtThisTimeStep();
  virtual void fillEssBC ();           // set boundary conditions
  virtual void mgFillEssBC(int grid_level);
  virtual void setIC();                // set u_prev equal to u(x,0)
  virtual void define (MenuSystem & menu,int level=MAIN);
  virtual void scan ();
  virtual void dump();

  real& valueNode(int node){return v->valueNode(node);}

  Mono1MG ();
  Mono1MG (String cellmodeltype);
 ~Mono1MG ();
  virtual void adm    (MenuSystem & menu);
  virtual void solveProblem();
  virtual void resultReport();

  // f = source, k = diffusion, I = initial cond., g = Dirichlet cond.
  virtual real f(const FiniteElement& fe, real t = DUMMY);
  virtual real k(const FiniteElement& fe, real t = DUMMY);
  virtual real I(const Ptv(real)& p);                     
  virtual real g(const Ptv(real)& p, real t=DUMMY);       
};
#endif
