#include <Beeler.h>



/* Copyright (c) 2003 by A BIG Corporation.  All Rights Reserved */

/***
   NAME
     Beeler
   PURPOSE
     
   NOTES
     
   HISTORY
     monicaha - Jun 23, 2003: Created.
***/

void Beeler :: defineStatic (MenuSystem& menu, int level)
{
  DPTRACE("Beeler::define");
  String description = "Parameters for the BeelerReuter model";
  String command = "Beeler";
  MenuSystem::makeSubMenuHeader (menu, description, command, level, 'P');
 
  menu.addItem (level, "Initial conditions", "init_cond",
		"Initial values",
		"-20.35 0.0000003 0.0241 0.01126 0.9871 1.0 0.0030 1.0 0.0", "S");

  menu.addItem (level, "gNa","gNa","max Na-current","4.0", "I1");
  menu.addItem (level, "eNa","eNa","equi potential","50.0", "I1");


  CellModel:: defineStatic (menu, level); 

}


void Beeler :: scan (MenuSystem& menu)
{
   DPTRACE("BeelerReuter::scan");

   y0.redim(evalSize());
   y0.scan(menu.get("Initial conditions"));
   gNa =  menu.get("gNa").getReal(); 
   eNa =  menu.get("eNa").getReal(); 
   CellModel::scan(menu);
}


void Beeler :: computeCurrents(const Vec(real)& y, real t) {
  real eS;

  V  = y(1);
  Ca = y(2);
  x  = y(3);
  m  = y(4);
  h  = y(5);
  j  = y(6);
  d  = y(7);
  f  = y(8);
  G  = y(9);

// Should not be needed:
  if (m>1) m=1;
  if (m<0) m=0;
  if (h>1) h=1;
  if (h<0) h=0;
  if (j>1) j=1;
  if (j<0) j=0;
  if (x>1) x=1;
  if (x<0) x=0;
  if (d>1) d=1;
  if (d<0) d=0;
  if (f>1) f=1;
  if (f<0) f=0;
 
 iK = 0.35*(4.0*(exp(0.04*(V+85.0))-1.0)/(exp(0.08*(V+53.0))+exp(0.04*(V+53.0)))+0.2*(V+23.0)/(1.0-exp(-0.04*(V+23.0))));
ix = x*0.8*(exp(0.04*(V+77.0))-1.0)/exp(0.04*(V+35.0));

iNa = (gNa*pow(m,3.0)*h*j+gNaC)*(V-eNa);
eS = -82.3-13.0287*log(Ca);
is = gs*d*f*(V-eS);

 stim_current = 0.0;

} 


void Beeler :: computeGateCoefs(const Vec(real)& y) {
  real V, c4, c5;
  V = y(1);
  c4 = 0.0; c5 = 0.0;
  ax = (0.0005*exp(0.0830*(V+50.0)) + c4*(V+c5))/(exp(0.057*(V+50.0))+1.0);
  bx = (0.0013*exp(-0.060*(V+20.0)) + c4*(V+c5))/(exp(-0.04*(V+20.0))+1.0);
  am = (0.0000*exp(0.0000*(V+47.0))-1.0*(V+47.0))/(exp(-0.1*(V+47.0))-1.0);
  bm = (40.000*exp(-0.056*(V+72.0)) + c4*(V+c5))/(exp(0.00*(V+72.0))+0);
  ah = (0.1260*exp(-0.250*(V+77.0)) + c4*(V+c5))/(exp(0.0*(V+77.0))+0);
  bh = (1.7000*exp(0.0000*(V+22.5))+c4*(V+c5))/(exp(-0.082*(V+22.5))+1.0);
  aj = (0.0550*exp(-0.250*(V+78.0)) + c4*(V+c5))/(exp(-0.2*(V+78.0))+1.0);
  bj = (0.3000*exp(0.0000*(V+32.0)) + c4*(V+c5))/(exp(-0.1*(V+32.0))+1.0);
  ad = (0.0950*exp(-0.010*(V-5.0))  + c4*(V+c5))/(exp(-0.072*(V-5.0))+1.0);
  bd = (0.0700*exp(-0.017*(V+44.0)) + c4*(V+c5))/(exp(0.05*(V+44.0))+1.0);
  af = (0.0120*exp(-0.008*(V+28.0)) + c4*(V+c5))/(exp(0.15*(V+28.0))+1.0);
  bf = (0.0065*exp(-0.020*(V+30.0)) + c4*(V+c5))/(exp(-0.2*(V+30.0))+1.0);
}


real Beeler :: totalCurrent() {
  return (iNa  + iK + ix + is + stim_current);
}


void Beeler :: setConstants() {
  eNa = 50.0;
  gNa = 4.0; 
  gNaC = 0.003; gs = 0.09;
  Cm = 1.0;
}


void Beeler :: setParameters(Ptv(real)& p)
{

  // cout << "resetting some paramers : " << p << endl;

  // dCai2 = p(5);
  T0 = p(4);
  tpeak = p(1);

  //stim_current = p(6);
  //stim_start = p(7);
  //stim_stop = p(8);
}

void Beeler :: evalStateDot (Vec(real)& dy, const DynamicState& s) {
  real itot;

  computeCurrents(s.yvec(),s.time());
  computeGateCoefs(s.yvec());
  itot = totalCurrent();

  dy(1) = -(1.0/Cm)*(itot);
  dy(2) = -1.0*pow(10.0,-7.0)*is+0.07*(pow(10.0,-7.0)-Ca);
  dy(3) = ax*(1-x)-bx*x;
  dy(4) = am*(1-m)-bm*m;
  dy(5) = ah*(1-h)-bh*h;
  dy(6) = 0; //aj*(1-j)-bj*j;
  dy(7) = ad*(1-d)-bd*d;
  dy(8) = af*(1-f)-bf*f;
  dy(9) = 0;
}
