#include <CellModel.h>
#include <createODEsolver.h>


void CellModel:: defineStatic (MenuSystem& menu, int level)
{
  DPTRACE("CellModel::define");
  ODE::defineStatic(menu,level+1);
  menu.addItem(level,"solver", "OdeSolver" ,
               "class name in OdeSolver hirarchy",
               "RungeKuttaCK45",
	       validationString(hierODEsolver())
               );

}

void CellModel:: scan (MenuSystem& menu)
{
  // Asumes that a menu item "Initial conditions" has been
  // defined in a subclass.   

  DPTRACE("CellModel::scan");
  
  ODE::scan(menu);
  ODEsolver_prm solv_prm;
  solv_prm.exp_sys.rebind(this); // explicit problem formulation
  solv_prm.solver_tp = menu.get("solver");
  attachSolver(*createODEsolver(solv_prm));
  plot_dt = 0;
  tip->setMinStepSize(10e-14);
  tip->modifyDelta(0.25);
  setTolerance(10e-4);

  //vectors used in the jacobian evaluation (for SDIRK solver)
  ydot1.redim(evalSize());
  ydot2.redim(evalSize());
  dF.redim(evalSize());
  tmp_state.redim(evalSize());
  h = 1.0e-8;
}


static TimeStepper internal_stepper;

void CellModel::forward(Ptv(real)& yy)
{ 
  int j;
  for (j=1; j <= yy.size(); j++)
    state().y(j) = yy(j);  

  real T0 = tipp->getTime();
  real dt = tipp->Delta();

  tip->setNewTime(T0);
  tip->setNewUpperTime(T0+dt);
  state().time() = T0;  

  evalStateDot((Vec(real)&)state().ydotvec(),state());
  ODE::timeLoopA();

  for (j=1; j <= yy.size(); j++)
    yy(j) = state().y(j);   
}

void CellModel::forward (Ptv(real)& yy, real T0, real dt, Ptv(real)& auxx )
{
  int j;
  for (j=1; j <= yy.size(); j++)
    state().y(j) = yy(j);

  tip->setNewTime(T0);
  tip->setNewUpperTime(T0+dt);

  state().time() = T0;

//cout << "inital: " << yy << endl;
  evalStateDot((Vec(real)&)state().ydotvec(),state());
//cout << "rhs: " << (Vec(real)&)state().ydotvec() << endl;

//cout << "using aux routine" << auxx(1) << endl;

//  ODE::timeLoopA_LR(auxx);
//ODE::timeLoopA();
  this->innerLoop(auxx);

// obs, endret her  for (j=1; j <= yy.size(); j++)
  for (j=1; j <= yy.size(); j++)
    //    if (j != V_index) kan vel ikke ha noe  si?
      yy(j) = state().y(j);
}

void CellModel::forward (Ptv(real)& yy, real T0, real dt)
{
  int j;

  for (j=1; j <= yy.size(); j++)
    state().y(j) = yy(j);

  tip->setNewTime(T0);
  tip->setNewUpperTime(T0+dt);

  state().time() = T0;

  evalStateDot((Vec(real)&)state().ydotvec(),state());

  ODE::timeLoopA();

  for (j=1; j <= yy.size(); j++)
    yy(j) = state().y(j);
}

void CellModel:: recomputeJacobian(Mat(real)& J, const DynamicState& y)
{
  int no_of_states = evalSize();

  evalStateDot(ydot1, y);
  for (int j = 1; j <= no_of_states; j++){
    tmp_state = y;
    tmp_state.y(j) += h;
    evalStateDot(ydot2,tmp_state);  
    dF.add(ydot2,'-',ydot1);
    dF.mult(1.0/h);
    for (int i = 1; i <= no_of_states; i++)
      J(i,j) = dF(i);      
  }  
}

void CellModel:: forwBackLU(Mat(real)& A, Vec(real)& x, Vec(real) b)
{
  A.forwBackLU(b,x);
}

void CellModel::print (Os os) 
{ 
  DPTRACE("CellModel::print");
  os << tipp->getTime() << " ";
  state().yvec().print(os); 
  os << '\n';
} 





