
                                                                 // -*- C++ -*-
// $Id: CellModel.h,v 1.1.1.1 2003/11/25 16:35:31 glennli Exp $
// prevent multiple inclusion of CellModel.h
#ifndef CellModel_h_IS_INCLUDED
#define CellModel_h_IS_INCLUDED

#include<Arrays_real.h>
#include <SaveSimRes.h>    // storage tool for later visualization
#include <ExplicitODEeqUDC.h>
#include <ODEsolver_prm.h>
#include <ODE.h>
// ------------------------- CellModel -----------------------

/*<CellModel:*/


class CellModel :  public ExplicitODEeqUDC, public ODE, public SimCase
{
protected:

  Handle(TimePrm)   tipp; 
  Vec(real)          y0;
  virtual void innerLoop(Ptv(real)& /*auxx*/) {
//    cout << "using default innerLoop\n"; 
    ODE::timeLoopA();
  }

  //vectors used in the Jacobian evaluation
  Vec(real) dF, ydot1, ydot2;
  DynamicState tmp_state;
  real h;
public:

  CellModel ()
  { DPTRACE("CellModel constructor"); }

 ~CellModel () {}

  virtual int evalSize() const { return 0;}
  virtual int evalNoDiscrite() const { return 0;}
  virtual int getTransmembraneIndex() const { return 1;}

  virtual void evalStateDot(Vec(real)& ydot, const DynamicState& s) = 0;

  virtual void setIV(DynamicState& s)
    { 
      for (int i=1;i<=s.yvec().size();i++)
	s.y(i) = y0(i); 

      s.time() = 0;
    }
 
  virtual real ionicCurrent(Vec(real)& /*yy*/, real /*t*/) {return 0; }

  virtual void setConstantJacobianElms(Mat(real)& J)
  {
    /*
      only in optimized sub-classes
    */
  }

  virtual void recomputeJacobian(Mat(real)& J, const DynamicState& y);
  virtual void forwBackLU(Mat(real)& A, Vec(real)& x, Vec(real) b);

  void attachTimer(TimePrm& globaltip)
   { tipp.rebind(globaltip); }


  virtual Ptv(real) currentState()              
    {
      DBP("er jeg her??");
      cout << evalSize() << endl;
      Ptv(real) silly(evalSize());
      for (int j = 1; j <= evalSize(); j++)
        silly(j) = y0(j);
      return silly;
    }
  

  void forward (Ptv(real)& yy);
  void forward (Ptv(real)& yy, real t0, real dt);
  void forward (Ptv(real)& yy, real t0, real dt, Ptv(real)& auxx);

  virtual void print (Os os); 

  virtual void scan (MenuSystem& menu);  // read parameters needed in equation
  static void defineStatic (MenuSystem& menu, int level = MAIN);
  
  virtual void define (MenuSystem& menu, int level = MAIN) 
  { defineStatic (menu,level); }

//  bool ok () const;             // is the simulator properly initialized?

  virtual void resultReport () {}  // write error norms to the screen
};
/*>CellModel:*/

/*Class:CellModel

NAME: CellModel - Defines the ODE describing the ionic currents

SYNTAX:	           @CellModel

KEYWORDS:          

   CellModel, ODE


DESCRIPTION:       

   

CONSTRUCTORS AND INITIALIZATION:


MEMBER FUNCTIONS:

"ionicCurrent" - returns the ioninc current in an evaluation point
in a fininte element. Called in integrands in Parabolic.

"evalStateDot" - defines the ODE system, called by ODE solver.

"evalSize" - returns the number of state variables

"scan" - reads and initializes ...

SEE ALSO:          

   Cells, FitzHugh, LuoRudy

DEVELOPED BY:   

AUTHOR:	        

   Glenn Terje Lines

End:
*/
#endif
/*
$Log: CellModel.h,v $
Revision 1.1.1.1  2003/11/25 16:35:31  glennli
INF-5610 oblig

*/


