#include <FK.h>


void FK:: defineStatic (MenuSystem& menu, int level)
{
  DPTRACE("FK::define");
  String description = "Parameters for the FK cell model";
  String command = "FK";
  MenuSystem::makeSubMenuHeader (menu, description, command, level, 'P');

  menu.addItem(level,"ode solver", "OdeSolver" ,
               "class name in OdeSolver hirarchy",
               "Euler1",
               //validationString(hierODEsolver())
               "Euler1"
               );

 
  CellModel:: defineStatic (menu, level); 
}

void FK:: scan (MenuSystem& menu)
{
  DPTRACE("FK::scan");
  
  //some of these should be made possible to choose from the menu
  v_c = 0.13;
  vg=0; 
  tgm1 = 18.2;
  tgm2 = 18.2;
  thm = 80;
  tgp = 0.10; // skal vre 10 men da blir det ikke full depolarisering
  thp = 1020;
  td = 1/5.8;
  to = 12.5;
  tr = 130;
  tsi= 127;
  k = 1; 
  vsi = 0.85;
 
  y0.redim(evalSize());
  y0.scan(menu.get("Initial conditions"));
 
 CellModel::scan(menu);

}

void FK:: evalStateDot (Vec(real)& dy, const DynamicState& s)
{
  real dg, dh, Jso, Jfi, Jsi, tgm;
  real v, g, h;

  v = s.y(1);
  g = s.y(2);
  h = s.y(3);

  if (v<v_c){
    Jfi = 0;
    Jso = v/to;
    if (v<vg)
      tgm = tgm1;
    else
      tgm = tgm2;
    dg = (1-g)*tgm;
    dh = (1-h)*thm;
  }
  else{
    Jfi = -(g/td)*(1-v)*(v-v_c);
    Jso = 1/tr;
    dg = -g*tgp;
    dh = -h*thp;
  }
  Jsi = -(h/tsi)*(1+tanh(k*(v-vsi))); 

  dy(1) = -Jfi - Jso - Jsi;
  dy(2) = dg; 
  dy(3) = dh; 
}

