#include<LuoRudy.h>

void LuoRudy:: defineStatic (MenuSystem& menu, int level)
{
  DPTRACE("LuoRudy::define");
  String description = "Parameters for the LuoRudy model";
  String command = "LuoRudy";
  MenuSystem::makeSubMenuHeader (menu, description, command, level, 'P');
 
  menu.addItem (level, "Initial conditions", "init_cond",
                "Initial values",
                "-40.0 10.0 145.0 16.065e-3 8.6 1.73 0.0 1.0 1.0 0.0 0.0 1.0", "S");

  menu.addItem (level, "External Na", "Nao",
                "Extracellular Sodium concentrations",
                "140", "I1"); 
  menu.addItem (level, "External Ca", "Cao",
                "Extracellular Calcium concentrations",
                "1.8", "I1"); 
  menu.addItem (level, "External K", "Ko",
                "Extracellular Potassium concentrations",
                "5.4", "I1");

  menu.addItem (level, "gNaMax", "gNaMax",
                "max Na-current",
                "16.0", "I1");

  menu.addItem (level, "gate_m_slow", "gate_m_slow",
                "gate_m_slow",
                "1.0", "I1");

  menu.addItem (level, "gate_h_slow", "gate_h_slow",
                "gate_h_slow",
                "1.0", "I1");

  CellModel:: defineStatic (menu, level); 

}


void LuoRudy:: scan (MenuSystem& menu)
{
   DPTRACE("LuoRudy::scan");

   y0.redim(evalSize());
   y0.scan(menu.get("Initial conditions"));

   Nao =  menu.get("External Na").getReal();  
   Cao =  menu.get("External Ca").getReal();  
   Ko  =  menu.get("External K").getReal();  
   gNaMax =  menu.get("gNaMax").getReal(); 
   gate_m_slow =  menu.get("gate_m_slow").getReal(); 
   gate_h_slow =  menu.get("gate_h_slow").getReal(); 

   CellModel::scan(menu);
  

}

void LuoRudy :: computeCurrents (const Vec(real)& y, real t)  
{
  real CSQN, aK1, gKMax, fCa, fNaK, frac, bK1, gK1Max, sigma;       
  real K1inf, t0, gRel, insKMax, Kp, insNaMax, dffCa, iCaKMax;        
  real eCaN, iCaMax, kLeak, eK, eK1, iCaNaMax, eNa, xi;          


  V     = y(1);
  Nai   = y(2);
  Ki    = y(3);
  Cai   = y(4);   Cai   = CaFree(y(4));
  CaJSR = y(5);   CaJSR = CaJSRFree(y(5));
  CaNSR = y(6);
  m     = y(7);
  h     = y(8);
  j     = y(9);
  x     = y(10);
  d     = y(11);
  f     = y(12);
 
//   for (int j = 1; j <= 12; j++)
//      cout << y(j) << " ";
//   cout << endl;     

// Should not be needed:
  if (m>1) m=1;
  if (m<0) m=0;
  if (h>1) h=1;
  if (h<0) h=0;
  if (j>1) j=1;
  if (j<0) j=0;
  if (x>1) x=1;
  if (x<0) x=0;
  if (d>1) d=1;
  if (d<0) d=0;
  if (f>1) f=1;
  if (f<0) f=0;
  
  
  real VFonRT;
  real expVFonRT;


  VFonRT = V*FonRT;
  expVFonRT = exp(VFonRT);

// II 	

// III a.


// y.print(cerr);
// cerr << Nao/Nai << " " << m << "\n";
  eNa = RTonF*log(Nao/Nai);
  iNa = gNaMax*pow(m,3.0)*h*j*(V-eNa);


// III b.
   
 
  fCa = 1.0/(1.0+pow(Cai/kmCa_IIIb,2.0));
  dffCa = d*f*fCa;
  if (abs(V)<1e-6) 
    {
      iCaMax = 5.4e-4*2*F*(1.0*Cai-0.341*Cao);
      iCaNaMax = 6.75e-7*F*(0.75*(Nai-Nao));
      iCaKMax  = 1.93e-7*F*(0.75*(Ki-Ko));
    } 
  else
    { 
      iCaMax = 5.4e-4*4*F*VFonRT*(1.0*Cai*exp(2.0*VFonRT)-0.341*Cao)/(exp(2.0*VFonRT)-1.0);
      iCaNaMax = 6.75e-7*F*VFonRT*(0.75*(Nai*expVFonRT-Nao))/(expVFonRT-1.0);
      iCaKMax  = 1.93e-7*F*VFonRT*(0.75*(Ki*expVFonRT-Ko))/(expVFonRT-1.0);
    }
  iCa = dffCa*iCaMax;
  iCaNa = dffCa*iCaNaMax;
  iCaK  = dffCa*iCaKMax;

// III c.

  gKMax = 0.282*sqrt(Ko/5.4);
// cerr << "log arg: " << Nai<<" "<<Nao<<" "<<Cai<<" "<<Cao<<" "<<Ki<<" "<<Ko<<" "<<pNaK <<"\n";
  eK = RTonF*log((Ko+pNaK*Nao)/(Ki+pNaK*Nai));
  xi = 1.0/(1.0+exp((V-56.26)/32.1));
  iK = gKMax*xi*x*x*(V-eK);

// III d.

  gK1Max = 0.75*sqrt(Ko/5.4);
  eK1 = RTonF*log(Ko/Ki);
  aK1 = 1.02/(1.0+exp(0.2385*(V-eK1-59.215)));
  bK1 = (0.49124*exp(0.08032*(V-eK1+5.476))+exp(0.06175*(V-eK1-594.31)))/(1.0+exp(-0.5143*(V-eK1+4.753)));
  K1inf = aK1/(aK1+bK1); 
  iK1 = gK1Max*K1inf*(V-eK1);

// III e.


  Kp = 1.0/(1.0+exp((7.488-V)/5.98));
  iKp = gKpMax*Kp*(V-eK1);

// III f.


  iNaCa = kNaCa*(1.0/(pow(kmNa,3.0)+pow(Nao,3.0)))*(1.0/(kmCa_IIIf+Cao))*(1.0/(1.0+ksat*exp((eta-1.0)*VFonRT)))*(exp(eta*VFonRT)*pow(Nai,3.0)*Cao-exp((eta-1.0)*VFonRT)*pow(Nao,3.0)*Cai);


// III g.


  sigma = (1.0/7)*(exp(Nao/67.3)-1.0);
  fNaK = 1.0/(1.0+0.1245*exp(-0.1*VFonRT)+0.0365*sigma*exp(-VFonRT));
  iNaK = iNaKMax*fNaK*(1/(1+pow(kmNai/Nai,1.5)))*(Ko/(Ko+kmKo));

// III h.   // ens(Ca)  ????
// III h.   // ens(Ca)  ????
 if (abs(V)<1e-6) 
    {
      insNaMax = 1.75e-7*F*(0.75*(Nai-Nao));
      insKMax  = 1.75e-7*F*(0.75*(Ki-Ko));
    }
 else
   {  
     insNaMax = 1.75e-7*F*VFonRT*(0.75*(Nai*expVFonRT-Nao))/(expVFonRT-1.0);
     insKMax  = 1.75e-7*F*VFonRT*(0.75*(Ki*expVFonRT-Ko))/(expVFonRT-1.0);
   }
  frac = 1.0+pow(kmnsCa/Cai,3.0);
  insNa = insNaMax/frac;
  insK  = insKMax/frac;

// III i.

  ipCa = ipCaMax*(Cai/(kmpCa+Cai));

// III j.

 
  eCaN = 0.5*RTonF*log(Cao/Cai);
  iCab = gCabMax*(V-eCaN);

// III k
  iNab = gNabMax*(V-eNa);

// III l, trengs ikke
// IV, trengs ikke

// V a.
//M holde ye med dV/dt for  bestemme t0, forelpig:

//  t = T0 + t; // global time
  t0 = tpeak + 2.0;;


// if ((t0>0)&&(dCai2>dCaith)) { 
 if (dCai2>dCaith) { 
   gRel = gRelMax_Va*(dCai2-dCaith)/(kmRel+dCai2-dCaith)*(1-exp(-(t-t0)/ton))*exp(-(t-t0)/toff);
   iRel = gRel*(CaJSR-Cai);   
  }
  else {
    iRel = 0; }

/*
cout << "t: " << t;
cout << " t-t0: " << t-t0;
cout << " AC " << dCai2;
cout << " CaJSR: " << CaJSR << " Cai: " << Cai;
cout << " irel: " << iRel << endl;
*/

// V c.

  CSQN = CSQNMax*(CaJSR/(CaJSR+kmCSQN));

// V b.

 
  if (CSQN>CSQNth) {
    gRel = gRelMax_Vb*(1-exp(-(t-t0)/ton))*exp(-(t-t0)/toff);
    cout << oform("CSQN: %f, CSQNth: %f \n", CSQN, CSQNth);
    cout << oform("iRel: %f, gRel*(CaJSR-Cai): %f \n", iRel, gRel*(CaJSR-Cai));


cout << oform(" V:%f, Nai:%f, Ki:%f, Cai:%f, CaJSR:%f, CaNSR:%f, m:%f, h:%f, j:%f, x:%f, d:%f, f:%f, y1:%f, y4:%f \n",V,Nai,Ki,Cai,CaJSR,CaNSR,m,h,j,x,d,f,y(1),y(4));     
//    errorFP("LuoRudy:: CSQN", " CSQN>CSQNth, yes really!");
 
//    iRel = iRel + gRel*(CaJSR-Cai);
  }

// V d.

 
  iUp = iUpMax*(Cai/(Cai+kmUp));

  kLeak = iUpMax/CaNSRMax;
  iLeak = kLeak*CaNSR;

// V e.

 
  iTr = (CaNSR-CaJSR)/Trtau;
  if ((t>=stim_start)&&(t<=stim_stop)) {
    ist = stim_current;
//    cout << oform("t: %f start: %f stop: %f cur: %f  ist: %f\n",t,stim_start,stim_stop, stim_current,ist);
  }	
  else
    ist = 0;
}

void LuoRudy :: computeGateCoefs (const Vec(real)& y)  
{
  real V, finf, ftau;


  V = y(1);

// III a.
  if (V>=-40) {
    ah = 0;
    bh = 1.0/(0.13*(1.0+exp((V+10.66)/(-11.1))));
    aj = 0;
    bj = 0.3*exp(-2.535e-7*V)/(1.0+exp(-0.1*(V+32.0))); 
  }
  else {
    ah = 0.135*exp((80.0+V)/(-6.8));
    bh = 3.56*exp(0.079*V)+3.1e5*exp(0.35*V);
    aj = (-1.2714e5*exp(0.2444*V)-3.474e-5*exp(-0.04391*V))*(V+37.78)/(1+exp(0.311*(V+79.23)));
    bj = 0.1212*exp(-0.01052*V)/(1.0+exp(-0.1378*(V+40.14))); 
  }
  am = 0.32*(V+47.13)/(1.0-exp(-0.1*(V+47.13)));
  if (V>-960.0)
    bm = 0.08*exp(-V/11.0);
  else
    bm = 0.08*exp(60.0/11.0);
// III b.
   

  // dinf = 1/z1, dtau = dinf*z2/z3, 
  // ad = dinf/dtau = dinf/(dinf*z2/z3) = z3/z2;
  ad = (0.035*(V+10))/(1.0-exp(-(V+10.0)/6.24));
  // bd = (1-dinf)/dtau = 1/dtau - dinf/dtau = ad/dinf - ad = ad*(z1-1); 
  bd = ad*exp(-(V+10.0)/6.24);
  finf = 1.0/(1+exp((V+35.06)/8.6))+0.6/(1+exp((50.0-V)/20.0));
  ftau = 1.0/(0.0197*exp(-pow(0.0337*(V+10.0),2.0))+0.02);
  af = finf/ftau;
  bf = (1-finf)/ftau;

// III c.


  ax = 7.19e-5*(V+30.0)/(1.0-exp(-0.148*(V+30.0)));
  bx = 1.31e-4*(V+30.0)/(-1.0+exp(0.0687*(V+30.0)));

}


real LuoRudy :: totalCurrent() 
{ return iNa + iCaNa + iNaCa + iNaK + iNab + insNa + iCaK + iK + iK1 + iKp + insK + iCa + ipCa + iCab + ist;
// return  iNatot + iKtot + iCatot;
}

void LuoRudy  :: setConstants() 
{
  DPTRACE("LuoRudy  :: setConstants");


  //vCell = pi*r^2*L;
  vCell = 38e-6;
  //aGeo = 2*pi*r^2+2*pi*r*L;
  aGeo = 0.767e-4;
  aCap = 2*aGeo;
  vMyo = vCell*0.68;
  vMito = vCell*0.26;
  vSR = vCell*0.06;
  vNSR = vCell*0.0552;
  vJSR = vCell*0.0048;
  vCelft = (vCell/0.88)*0.12;

  R = 0.08554*965.0e2;
  T = 273.0+37.0;
  F = 96500.0;
  RTonF = R*T/F;
  FonRT = F/(R*T);

//  gNaMax = 16.0;
//  gNaMax = 8.0;
  pNaK = 0.01833;
  gKpMax = 0.0183;
  kNaCa = 2000;
  kmNa = 87.5;
  kmCa_IIIb = 0.6*0.001;
  kmCa_IIIf = 1.38;
  ksat = 0.1;
  eta = 0.35;
  iNaKMax = 1.5;
  kmNai = 10.0;
  kmKo = 1.5;
  kmnsCa = 1.2*0.001;
  ipCaMax = 1.15;
  kmpCa = 0.5*0.001;
  gCabMax = 0.003016;
  gNabMax = 0.00141;
  dCaith = 0.18*0.001;
  gRelMax_Va = 60.0;
  kmRel = 0.8e-3;
  ton = 2.0;
  toff = 2.0;
  dCai2 = 0; // for now
  CSQNMax = 10;
  kmCSQN = 0.8; 
  gRelMax_Vb = 4.0;
  CSQNth = 0.7*CSQNMax; // or higher...
  iUpMax = 0.005;
  kmUp = 0.92*0.001; 
  CaNSRMax = 15.0;
  Trtau = 180.0;
  T0 = 0.0; 
  tpeak = 0.0;
}

void LuoRudy :: setParameters(Ptv(real)& p)
{

  // cout << "resetting some paramers : " << p << endl;

  dCai2 = p(5);
  T0 = p(4);
  tpeak = p(1);

  stim_current = p(6);
  stim_start = p(7);
  stim_stop = p(8);

}

real LuoRudy :: CaFree(real CaTot) 
{
  real K1 = 0.5;
  real K2 = 2.38;
  real M1 = 70.0;
  real M2 = 50.0;

  real T = CaTot*1000.0;

  real b = K1 + K2 + M1 + M2 - T;
  real c = K1 * K2 + K2 * M1 + K1 * M2 - K1 * T - K2 * T;
  real d = -(K1 * K2 * T);

  real p = (-(1.0/3.0)*(b*b)+c)/3.0;
  real q = ((2.0/27.0)*(b*b*b) - (1.0/3.0)*b*c + d)/2.0;

  real v = acos(-q/sqrt(-p*p*p));
  real y1 = 2.0*sqrt(-p)*cos(v/3.0);

  real Ca_free = y1 - b/3.0;

  return Ca_free*0.001;

}

real LuoRudy :: CaJSRFree(real CaJSRTot) 
{
  real M = 10.0;
  real k = 0.8;
  real T = CaJSRTot;
  real d = T-k-M;
  real CaJSR = 0.5*(d+sqrt(d*d+4.0*k*T));

  return CaJSR;

}


//void LuoRudy :: evalStateDot (Vec(real)& dy, const Vec(real)& y, real t)  
void LuoRudy :: evalStateDot (Vec(real)& dy, const DynamicState& s)  
{
//  DPTRACE("LuoRudy :: evalStateDot");
  real itot, iNatot, iKtot, iCatot, iCaNSRtot, iCaJSRtot, Cm;

//cout << "state: ";
//  s.yvec().print(cout);
//cout << "\n";
// cout << y(1) << "\n";

  computeCurrents(s.yvec(),s.time());
  computeGateCoefs(s.yvec());
  

//  itot = totalCurrent();
  iNatot = iNa + iCaNa + 3*iNaCa + 3*iNaK + iNab + insNa;
  iKtot  = iCaK + iK + iK1 + iKp - 2*iNaK + insK;
  iCatot = iCa - 2*iNaCa + ipCa + iCab;
  itot = iNatot + iKtot + iCatot + ist;
  iCaNSRtot = iUp - iLeak - iTr;
  iCaJSRtot =  -iRel + iTr;




  Cm = 1.0;
  dy(1) = -0.0*(1.0/Cm)*(itot);

  dy(2) = -(iNatot*aCap)/(vMyo*F);
  dy(3) = -(iKtot*aCap)/(vMyo*F);
  dy(4) = -(iCatot*aCap)/(vMyo*2*F) - ((iUp-iLeak)*(vNSR/vMyo) - iRel*(vJSR/vMyo));

//  dy(5) = -(iCaJSRtot*aCap)/(vJSR*F);
//  dy(6) = -(iCaNSRtot*aCap)/(vNSR*F);

dy(5) =  -iRel + iTr;
dy(6) =  iUp - iLeak - iTr*(vJSR/vNSR);


  dy(7)  = gate_m_slow*(am*(1.0-m)-bm*m);
  dy(8)  = gate_h_slow*(ah*(1.0-h)-bh*h);
  dy(9)  = (aj*(1.0-j)-bj*j);
  dy(10) = (ax*(1.0-x)-bx*x);
  dy(11) = (ad*(1.0-d)-bd*d);
  dy(12) = (af*(1.0-f)-bf*f);

// cout << "t: " << t << "\n";
// 
//cout << "state derived: ";
//   dy.print(cout);
//cout << "\n";
}
