
                                                                 // -*- C++ -*-
// $Id: LuoRudy.h,v 1.1.1.1 2003/11/25 16:35:31 glennli Exp $
#ifndef LuoRudy_h_IS_INCLUDED
#define LuoRudy_h_IS_INCLUDED

#include<math.h>
#include<CellModel.h>
#include<ODEsolver_prm.h>

// ------------------------- LuoRudy -----------------------

/*<LuoRudy:*/

class LuoRudy : public CellModel
{
protected:

  real T0, tpeak;
  real V, Nai, Ki, Cai, CaJSR, CaNSR, m, h, j, x, d, f;
  real Cao, Nao, Ko;
  real iNa,iCa,iK,iNaCa,iRel,iUp,iLeak,iTr,iCaK,iCaNa;
  real iK1,iKp,iNaK,ipCa,iNab,iCab,insNa,insK;
  real am,bm,ah,bh,aj,bj,ax,bx,ad,bd,af,bf;
  real vCell, aGeo, aCap, vMyo, vMito, vSR, vNSR, vJSR, vCelft; 
  real R,T,F,RTonF,FonRT;

  real gNaMax, kmCa_IIIb,kmCa_IIIf, pNaK, gKpMax, kNaCa, kmNa, kmCa, ksat;
  real eta, iNaKMax, kmNai, kmKo, kmnsCa, ipCaMax, kmpCa, gCabMax, gNabMax;
  real dCaith, gRelMax_Va, kmRel, ton, toff, dCai2, CSQNMax, kmCSQN;
  real gRelMax_Vb, CSQNth, iUpMax, kmUp, CaNSRMax, Trtau;

  real gate_m_slow, gate_h_slow; 
  real stim_start,stim_stop,stim_current,ist;

  void computeCurrents(const Vec(real)& y, real t);
  void computeGateCoefs(const Vec(real)& y);
  real totalCurrent();
  void setConstants();

  real CaFree(real CaTot);
  real CaJSRFree(real CaJSRTot);

  virtual void innerLoop(Ptv(real)& auxx) {ODE::timeLoopA_LR(auxx);}

public:

  LuoRudy ()
  {
    DPTRACE("LuoRudy constructor"); 
    setConstants();
  }

 ~LuoRudy () {}

//  void evalStateDot (Vec(real)& f, const Vec(real)& y, real t)  ;
  virtual void evalStateDot(Vec(real)& ydot, const DynamicState& s);

  real ionicCurrent(Vec(real)& y, real t) 
  { 
    computeCurrents(y,t);
    return totalCurrent();
  }

  int evalSize() const {return 12;}

  static void defineStatic (MenuSystem& menu, int level = MAIN);

  void define (MenuSystem& menu, int level = MAIN) 
     {defineStatic (menu,level);}

  virtual void scan   (MenuSystem& menu); 
  virtual void setParameters(Ptv(real)& p );
//  void resultReport ();  
};
/*>LuoRudy:*/

/*Class:LuoRudy

NAME: LuoRudy - Implements the ionic model of Luo and Rudy

SYNTAX:	           @LuoRudy

KEYWORDS:          

   LuoRudy, CellModel


DESCRIPTION:       

   Sub class of CellModel

CONSTRUCTORS AND INITIALIZATION:

   The constructor sets some constants.

MEMBER FUNCTIONS:

"scan" - reads and initializes ...


EXAMPLE:           


SEE ALSO:          


DEVELOPED BY:   

AUTHOR:	        

   Glenn Terje Lines

End:
*/
#endif
/*
$Log: LuoRudy.h,v $
Revision 1.1.1.1  2003/11/25 16:35:31  glennli
INF-5610 oblig

*/


