#ifndef SDIRK_h_is_included
#define SDIRK_h_is_included

#include <FEM.h>
#include <HandleId.h>
#include <Winslow.h>
#include <DynamicState.h>

class Winslow;

class SDIRK
//4 step SDIRK method of order three
{ 
 protected:
  DynamicState current_state, internal_state;

  real local_t;                          //the current local time
  real prev_dt;                          //the previous (local) step size
  real min_dt;                           //minimum time step
  real loc_error, tol, s_fac;
  real eta, etamin, kappa;               //control parameters for Newton its

  bool new_jacobian, first_step;

  int its, work, maxits, rejects;

  real gamma, a21, b1_, b2_, b1, b2, b3; //method coefficients
  real c2, c3;                           //method coefficients   
  real e1, e2, e3;

  Mat(real) J;                           //the Jacobi matrix  
  Mat(real) A;                           //I-h*gamma*J, used in the Newton its.

  Vec(real) y2, y3;                      //stage values 
  Vec(real) z2, z3, z4;                  //zi=yi-y0
  Vec(real) k1, k2, k3, k4;
  Vec(real) dk, prev_dk, b_vec;
  Vec(real) temp, error, prev_steps;
  //vectors used in the (slow) Jacobian evaluation
  //Vec(real) dF, ydot1, ydot2;
  //real h;      //parameter used to compute the Jacobian matrix with differences 

  int no_of_states;                      //the problem size
  Winslow *problem;                    //specification of the IVP 

  void reduceTimeStep(real& delta_t);
  void computeFirstJacobian();
  void recomputeJacobian();
  void computeNewA(const real& local_dt);
 
  virtual bool computeStageVal
       (Vec(real)& z, const Vec(real)& prev_steps, const real& dt);
  virtual void advanceOneStep(Vec(real)& y, real& local_dt);
  virtual bool doStep(Vec(real)& y, const real& dt);


  inline void problemEvalStateDot(Vec(real)& ydot,const DynamicState& state);
  //{ problem->evalStateDot(ydot,state); /*work++;*/ }

 

 public:
  SDIRK() {}
  virtual ~SDIRK(){}
  int getWork(){return work;}
  int getRejects(){return rejects;}
  virtual void advance
          (Vec(real)& y, real global_t, real global_dt);
  virtual void attachAndInit(Winslow* prob); 

};

#endif
