#include "Winslow.h"

Winslow::Winslow()
{
  DPTRACE("Winslow::constructor");
  setConstants();
  y.redim(evalSize());
  dy.redim(evalSize());
  dF.redim(evalSize());
  tmp.redim(evalSize());
  ydot1.redim(evalSize());
  ydot2.redim(evalSize());
  ydot3.redim(evalSize());
  diff = 1.0e-8;
  diffinv = 1/diff;

}


void Winslow :: innerLoop(Ptv(real)& /*auxx*/) {

  if (!sdirk)
    ODE::timeLoopA();
  else
    {
      Vec(real) state_vec(31);
      for (int i = 1; i <= 31; i++)
	state_vec(i) = state().y(i);
      
      real T0 = tip->getCurrentTime();
      real dt = tip->getUpperTime() - T0;

      SDIRKsolver->advance(state_vec, T0, dt);

      for (int i = 1; i <= 31; i++)
	state().y(i)= state_vec(i);
    }
}



void Winslow:: defineStatic (MenuSystem& menu, int level)
{
  DPTRACE("Winslow::define");
  String description = "Parameters for the Winslow model";
  String command = "Winslow";
  MenuSystem::makeSubMenuHeader (menu, description, command, level, 'P');
 
  menu.addItem (level, "Initial conditions", "init_cond",
                "Initial values",
                "2.4676e-4 0.99869 0.99887 0.6935 1.4589e-4 3.742e-5 1 159.48 8.464e-5 0.2620 0.2616 0.4929 6.027e-4 2.882e-9 0.5065 0.99802 1.9544e-6 0 0 0 1.9734e-3 0 0 0 0 0 0.7959 5.5443e-3 136.64e-3 1.315e-4 -35", "S");

  menu.addItem (level, "External Na", "Nao",
                "Extracellular Sodium concentrations",
                "138.0", "R1"); 
  menu.addItem (level, "External Ca", "Cao",
                "Extracellular Calcium concentrations",
                "2.0", "R1"); 
  menu.addItem (level, "External K", "Ko",
                "Extracellular Potassium concentrations",
                "4.0", "R1");  
  menu.addItem (level, "SDIRKsolver", "sdirk",
                "1 for SDIRKsolver",
                "1", "R1");

  CellModel:: defineStatic (menu, level); 
}

void Winslow:: scan (MenuSystem& menu)
{
   DPTRACE("Winslow::scan");
   
   y0.redim(evalSize());
   y0.scan(menu.get("Initial conditions"));

   Nao =  menu.get("External Na").getReal();  
   Cao =  menu.get("External Ca").getReal();  
   Ko  =  menu.get("External K").getReal();  
   sdirk = (menu.get("SDIRKsolver").getInt()==1);

   CellModel::scan(menu);
   //   sdirk = true;
   if (sdirk) {
     SDIRKsolver = new SDIRK;
     SDIRKsolver->attachAndInit(this);
   }


}

void Winslow :: setParameters(Ptv(real)& p)
{

  //cout << "resetting some paramers : " << p << endl;

  stim_current = p(6);
  stim_start = p(7);
  stim_stop = p(8);

}

void Winslow :: setConstants() 
{
  //standard ionic concentrations
  Nao = 138.0;
  Cao = 2.0;
  Ko = 4.0;
  Nai = 10.0;

  //Cell geometry parameters
  cSC  = 1.00;
  aCap = 1.534e-4;
  vMyo = 25.84e-6;
  vJSR = 0.16e-6;
  vNSR = 2.1e-6;
  vSS  = 1.2e-9;

  //Membrane current parameters
  F       = 96.500;
  T       = 310;
  R       = 8.314; 
  RTonF   = R*T/F;
  FonRT   = F/(R*T);
  gKrMax = 0.0034;
  gKsMax = 0.00271;
  gto1Max = 0.23815;
  gK1Max = 2.8;
  gKpMax = 0.002216;
  gNaMax = 12.8;
  kNaCa  = 0.30;
  KmNa   = 87.5;
  KmCa   = 1.38;
  KmK1   = 13.0;
  ksat   = 0.2;
  eta     = 0.35;
  iNaKMax= 0.693;
  KmNai  = 10.0;
  KmKo   = 1.5;
  ipCaMax= 0.05;
  KmpCa  = 0.00005;
  gCabMax= 0.0003842;
  gNabMax= 0.0031; 

  //SR parameters
  v1RyR   = 1.8;
  Kfb     = 0.168e-3;
  Krb     = 3.29;
  KSR     = 1.0;
  Nfb     = 1.2;
  Nrb     = 1.0;
  vmaxf   = 0.813e-4;
  vmaxr   = 0.318e-3;
  tautr   = 0.5747;
  tauxfer = 26.7;
  kaplus  = 0.01215;
  kaminus = 0.576;
  kbplus  = 0.00405;
  kbminus = 1.930;
  kcplus  = 0.100;
  kcminus = 0.0008;
  ncoop   = 4;
  mcoop   = 3;

  //L-type Ca Channel parameters
  fL      = 0.3;
  gL      = 2.0;
  bL      = 2.0;
  aL      = 2.0;
  omegaL  = 0.01;
  Pscale  = 1.0;
  PCaL    = Pscale*3.125e-4;
  PKL     = Pscale*5.79e-7;
  iCahalf = -0.265;

  //Buffering parameters
  LTRPNtot    = 70e-3;
  HTRPNtot    = 140e-3;
  khtrpnplus  = 20.0;
  khtrpnminus = 66.0e-6;
  kltrpnplus  = 40.0;
  kltrpnminus = 0.040;
  CMDNtot     = 50e-3;
  CSQNtot     = 15.0;
  EGTAtot     = 10.0;
  KmCMDN      = 2.38e-3;
  KmCSQN      = 0.8;
  KmEGTA      = 0.15e-3;

  fKo  = sqrt(Ko/4.0);

};

void Winslow::computeCurrents (const Ptv(real)& state, real time)
{
  real eKs, Kp, sigma, fNaK, eCa;

  m       = state(1);
  h       = state(2);
  j       = state(3);
  xKr     = state(4);
  xKs     = state(5);
  xto1    = state(6);
  yto1    = state(7);
  Ki      = state(8);
  Cai     = state(9);
 
  open    = state(26);
  yL      = state(27);
  V       = state(31);

  VFonRT = V*FonRT;
  expVFonRT = exp(VFonRT);

  //Na+ current iNa
  eNa = RTonF*log(Nao/Nai);
  iNa = gNaMax*pow(m,3.0)*h*j*(V-eNa);

  //Rapid-activating delayed rectifier K+ current iKr
  eK  = RTonF*log(Ko/Ki);
  rV  = 1.0/(1.0+1.4945*exp(0.0446*V));
  iKr = gKrMax*fKo*rV*xKr*(V-eK);

  //Slow-activating delayed rectifier K+ current iKs
  eKs = RTonF*log((Ko+0.01833*Nao)/(Ki+0.01833*Nai));
  iKs = gKsMax*pow(xKs,2.0)*(V-eKs);

  //Transient outward K+ current ito1
  ito1 = gto1Max*xto1*yto1*(V-eK);

  //Time-Independent K+ current iK1
  K1inf = 1.0/(2.0+exp(1.5*(V-eK)*FonRT));
  iK1   = gK1Max*K1inf*(Ko/(Ko+KmK1))*(V-eK);

  //Plateau current iKp
  Kp  = 1.0/(1.0+exp((7.488-V)/5.98));
  iKp = gKpMax*Kp*(V-eK);

  //NCX Current iNaCa
  iNaCa = kNaCa*(5000.0/(pow(KmNa,3.0)+pow(Nao,3.0)))*(1.0/(KmCa+Cao))
         *(1.0/(1.0+ksat*exp((eta-1)*V*FonRT)))*(exp(eta*VFonRT)
         *pow(Nai,3.0)*Cao-exp((eta-1)*VFonRT)*pow(Nao,3.0)*Cai);

  //Na+-K+ pump current iNaK
  sigma = (1.0/7.0)*(exp(Nao/67.3)-1.0);
  fNaK = 1.0/(1.0+0.1245*exp(-0.1*VFonRT)+0.0365*sigma*exp(-VFonRT));
  iNaK = iNaKMax*fNaK*1/(1.0+pow(KmNai/Nai,1.5))*Ko/(Ko+KmKo);
  
  //Sarcolemmal Ca2+ pump current ipCa
  ipCa = ipCaMax*Cai/(KmpCa+Cai);

  //Ca2+ background current ibCa
  eCa  = RTonF/2*log(Cao/Cai);
  iCab = gCabMax*(V-eCa);

  //Na+ background current ibNa
  iNab = gNabMax*(V-eNa);

  //currents through L-type Ca2+ channels 
  iCaMax = (PCaL/cSC)*(4.0*1000.0*F*VFonRT)*
            (0.001*exp(2.0*VFonRT)-0.341*Cao)/(exp(2*VFonRT)-1.0);
  iCa = iCaMax*yL*open;
  PKLprime = PKL/(1.0+(min(iCaMax,0.0)/iCahalf));
  iCaK = PKLprime/cSC*yL*open*1000*F*VFonRT*(Ki*expVFonRT-Ko)/(expVFonRT-1);


  //cout << "stim_start = " << stim_start << " stim_stop = " << stim_stop << endl;
  if ((time>=stim_start)&&(time<=stim_stop)) {
    ist = stim_current;
  }	
  else
    ist = 0;
}

void Winslow::computeGateCoefs(const Ptv(real)& state)
{
  /*computing gate coefficients for sodium and potassium currents */

  real k12, k21;

  V = state(31);

  if (abs(V+47.13)<=1e-3) //Taylor expansion close to -47.13
    alpham = 1.0/(0.1-0.005*(V+47.13));
  else
    alpham = 0.32*(V+47.13)/(1-exp(-0.1*(V+47.13)));
  
  betam = 0.08*exp(-V/11.0);

  if (V>=-40.0)
    alphaj = 0.0;
  else alphaj = (-1.2714e5*exp(0.2444*V)-3.474e-5*exp(-0.04391*V))
	 *(V+37.78)/(1+exp(0.311*(V+79.23)));

  if (V>=-40.0)
    betaj = 0.3*exp(-2.535e-7*V)/(1+exp(-0.1*(V+32)));
  else betaj = 0.1212*exp(-0.01052*V)/(1.0+exp(-0.1378*(V+40.14)));
  
  if (V>=-40.0)
    alphah = 0.0;
  else 
    alphah = 0.135*exp((80.0+V)/(-6.8));

  if (V>=-40.0)
      betah = 1.0/(0.13*(1+exp((V+10.66)/(-11.1))));
  else betah = 3.56*exp(0.079*V)+3.1e5*exp(0.35*V);
 
  /*
  if (V >= -90.0)
     dm = am*(1-m)-bm*m;
  else{
     dm = 0.0;
     m = am/(am+bm);
  }
  */

  //Rapid-activating delayed rectifier K+ current iKr
  k12 = exp(-5.495+0.1691*V);
  k21 = exp(-7.677-0.0128*V);
  xKrinf = k12/(k12+k21);
  tauxKr = 1.0/(k12+k21) + 27.0;

  //Slow-activating delayed rectifier K+ current iKs
  xKsinf = 1.0/(1+exp(-(V-24.7)/13.6));
  real a1, a2;
  a1 = 7.19e-5*(V-10)/(1-exp(-0.148*(V-10)));
  a2 = 1.31e-4*(V-10)/(exp(0.0687*(V-10))-1);
  tauxKs = 1.0/(a1+a2);

  //Transient outward K+ current ito
  alphaxto1 = 0.04516*exp(0.03577*V);
  betaxto1 = 0.0989*exp(-0.06237*V);
  alphayto1 = (0.005415*exp(-(V+33.5)/5.0))/
    (1.0+0.051335*exp(-(V+33.5)/5.0));
  betayto1 = (0.005415*exp((V+33.5)/5.0))/(1.0+0.051335*exp((V+33.5)/5.0));

  yInf = 0.8/(1+exp((V+12.5)/5.0))+0.2;
  tauy = 20.0+600.0/(1+exp((V+20)/9.5));
} 

void Winslow::evalStateDot(Vec(real)& ydot, const DynamicState& s)
{ 
  real C1_to_CCa1, C2_to_CCa2, C3_to_CCa3, C4_to_CCa4;
  real CCa1_to_C1, CCa2_to_C2, CCa3_to_C3, CCa4_to_C4;
  real dc0, dc1, dc2, dc3, dc4, dopen; 
  real dcCa0, dcCa1, dcCa2, dcCa3, dcCa4; 
  real dpc1, dpo1, dpc2, dpo2;

  real dLTRPNCa, dHTRPNCa, jTrpn;

  CaNSR   = s.y(10);
  CaJSR   = s.y(11);
  pc1     = s.y(12);
  po1     = s.y(13);
  po2     = s.y(14);
  pc2     = s.y(15);
  c0      = s.y(16); 
  c1      = s.y(17);
  c2      = s.y(18);
  c3      = s.y(19);
  c4      = s.y(20);

  cCa0    = s.y(21);
  cCa1    = s.y(22);
  cCa2    = s.y(23);
  cCa3    = s.y(24);
  cCa4    = s.y(25);

  LTRPNCa = s.y(28);
  HTRPNCa = s.y(29);
  Cass    = s.y(30);

  computeCurrents(s.yvec(),s.time());
  computeGateCoefs(s.yvec());
 
  //RyR Channel
  jRel = v1RyR*(po1+po2)*(CaJSR-Cass);

  //SERCA2a Pump 
  fb = pow(Cai/Kfb,Nfb);
  rb = pow(CaNSR/Krb,Nrb);
  jUp = KSR*(vmaxf*fb-vmaxr*rb)/(1+fb+rb);

  //Intracellular Ca fluxes
  jTr = (CaNSR-CaJSR)/tautr;
  jXfer = (Cass-Cai)/tauxfer;

  //a1 = kltrpnminus * LTRPNCa;
  dLTRPNCa = kltrpnplus*Cai*(1.0 - LTRPNCa) - kltrpnminus * LTRPNCa;

  //a1 = khtrpnminus * HTRPNCa;
  dHTRPNCa = khtrpnplus*Cai*(1.0 - HTRPNCa) - khtrpnminus * HTRPNCa;
  
  jTrpn = LTRPNtot*dLTRPNCa+HTRPNtot*dHTRPNCa;


  //L-type Ca2+ current iCa
  alphaL      = 0.4*exp((V+2.0)/10.0);
  betaL       = 0.05*exp(-(V+2.0)/13.0);
  alphaprimeL = alphaL * aL;
  betaprimeL  = betaL/bL;
  gammaL = 0.10375*Cass;
 
  C1_to_CCa1 = aL*gammaL;	
  C2_to_CCa2 = aL*C1_to_CCa1;	
  C3_to_CCa3 = aL*C2_to_CCa2;	
  C4_to_CCa4 = aL*C3_to_CCa3;	
		
  CCa1_to_C1 = omegaL/bL;	
  CCa2_to_C2 = CCa1_to_C1/bL;	
  CCa3_to_C3 = CCa2_to_C2/bL;	
  CCa4_to_C4 = CCa3_to_C3/bL;	

  dc0 = betaL*c1+omegaL*cCa0-(4*alphaL+gammaL)*c0;
  dc1 = 4.0*alphaL*c0+2.0*betaL*c2+CCa1_to_C1*cCa1-(betaL+3.0*alphaL+C1_to_CCa1)*c1;
  dc2 = 3.0*alphaL*c1 + 3.0*betaL*c3 + CCa2_to_C2*cCa2-(2.0*betaL+2.0*alphaL+C2_to_CCa2)*c2;
  dc3 = 2.0*alphaL*c2 + 4.0*betaL*c4 + CCa3_to_C3*cCa3-(3.0*betaL+alphaL+C3_to_CCa3)*c3; 
  dc4 = alphaL*c3 + gL*open + CCa4_to_C4*cCa4 - (4.0*betaL+fL+C4_to_CCa4)*c4;
 
  dcCa0 = betaprimeL*cCa1 + gammaL*c0 - (4.0*alphaprimeL+omegaL)*cCa0;
  dcCa1 = 4.0*alphaprimeL*cCa0 + 2.0*betaprimeL*cCa2 + C1_to_CCa1*c1 
         -(betaprimeL+3.0*alphaprimeL+CCa1_to_C1)*cCa1; 
  dcCa2 = 3.0*alphaprimeL*cCa1 + 3.0*betaprimeL*cCa3 + C2_to_CCa2*c2
         -(2.0*betaprimeL+2.0*alphaprimeL+CCa2_to_C2)*cCa2;
  dcCa3 = 2.0*alphaprimeL*cCa2 + 4.0*betaprimeL*cCa4 + C3_to_CCa3*c3
    - (3.0*betaprimeL+alphaprimeL+CCa3_to_C3)*cCa3;
  dcCa4 = alphaprimeL*cCa3 + C4_to_CCa4*c4 - (4.0*betaprimeL+CCa4_to_C4)*cCa4;

  dopen =  fL*c4 - gL*open;

  //RyR Channel
  pow_m = pow((Cass*1000.0),mcoop);
  pow_n = pow((Cass*1000.0),ncoop);

  dpc1 = -kaplus*pow_n*pc1+kaminus*po1;
  dpo2 =  kbplus*pow_m*po1 - kbminus*po2;
  dpc2 =  kcplus*po1 - kcminus*pc2; 
  dpo1 = -(dpc1 + dpo2 + dpc2);

  
  betass = beta_ss(Cass);
  betaJSR = beta_JSR(CaJSR);
  betai = beta_i(Cai);
  

  ydot(1) = alpham*(1-m)-betam*m;  //dm;
  ydot(2) = alphah*(1-h)-betah*h;
  ydot(3) = alphaj*(1-j)-betaj*j;
  ydot(4) = (xKrinf-xKr)/tauxKr;
  ydot(5) = (xKsinf-xKs)/tauxKs;
  ydot(6) = alphaxto1*(1.0-xto1)-betaxto1*xto1;
  ydot(7) = alphayto1*(1.0-yto1)-betayto1*yto1;
  ydot(8) = -(iKr+iKs+ito1+iK1+iKp+iCaK-2*iNaK)*aCap*cSC/(vMyo*1000.0*F);
  ydot(9)= betai*(jXfer-jUp-jTrpn-(iCab-2*iNaCa+ipCa)*
  aCap*cSC/(2*vMyo*1000*F));
  ydot(10)= jUp*vMyo/vNSR-jTr*vJSR/vNSR;
  ydot(11)= betaJSR*(jTr-jRel);
  ydot(12)= dpc1;
  ydot(13)= dpo1;
  ydot(14)= dpo2;
  ydot(15)= dpc2;
  ydot(16)= dc0;
  ydot(17)= dc1;
  ydot(18)= dc2;
  ydot(19)= dc3;
  ydot(20)= dc4;
  ydot(21)= dcCa0;
  ydot(22)= dcCa1;
  ydot(23)= dcCa2;
  ydot(24)= dcCa3;
  ydot(25)= dcCa4;
  ydot(26)= dopen;
  ydot(27)= (yInf-yL)/tauy;
  ydot(28)= dLTRPNCa;
  ydot(29)= dHTRPNCa; 
  ydot(30)= betass*(jRel*vJSR/vSS-jXfer*vMyo/vSS-iCa*aCap*
          cSC/(2*vSS*1000*F));
  ydot(31) = -(iNa+iCa+iCaK+iKr+iKs+ito1
         +iK1+iKp+iNaCa+iNaK+ipCa+iCab+iNab+ist);  
}

real Winslow :: ionicCurrent(Vec(real)& y, real t)
{
  DPTRACE("Winslow::ionicCurrent");
  
  computeCurrents(y,t);
  return totalCurrent();
}


real Winslow::totalCurrent()
{
  return iNa+iCa+iCaK+iKr+iKs+ito1+iK1+iKp+iNaCa+iNaK+ipCa+iCab+iNab+ist;
}


