#ifndef Winslow_h_IS_INCLUDED
#define Winslow_h_IS_INCLUDED

#include <CellModel.h>
#include <SDIRK.h>


class SDIRK;

class Winslow : public CellModel
{

protected:
	
  //state varibles
  real V, m, dm, h, j, xKr, xKs, xto1, yto1, Ki, Cai, CaNSR, Cass, CaJSR;
  real pc1, po1, pc2, po2, c0, c1, c2, c3, c4, open;
  real cCa0, cCa1, cCa2, cCa3, cCa4, yL, LTRPNCa, HTRPNCa;

  /*ionic currents and variables used in both computeDerivatives
    and recomputeJacobian*/
  real fKo, rV, PKLprime, VFonRT, expVFonRT, fb, rb;
  real jXfer, jUp, jTr, jTrpn, iCab, iNaCa, ipCa, iCa, jRel, eNa; 
  real iCaMax, eK, K1inf, betai, betass, betaJSR;
  real alphah, betah, alphaj, betaj, alpham, betam; 
  real tauxKr, tauxKs, tauy, alphaxto1, betaxto1, alphayto1, betayto1; 
  real pow_m, pow_n;
  real alphaL, betaL, alphaprimeL, betaprimeL, gammaL;

  //used in the forwBackLU function
  Vec(real) y;
  //vectors used in computing the derivatives and the Jacobian
  Vec(real) dy, dF, ydot1, ydot2, ydot3;
  DynamicState tmp;
  real diff, diffinv;

  //membrane currents
  real iNa,iCaK,iKr,iKs,ito1,iK1,iKp,iNaK,iNab;

  //gate coefficients
  real xKrinf, xKsinf, yInf;

  real cSC, aCap, vMyo, vJSR, vNSR, vSS;
  real F, T, R, RTonF, FonRT, gKrMax,gKsMax, gto1Max, gK1Max, gKpax;
  real gNaMax, kNaCa, KmNa, KmCa, KmK1, ksat, eta, iNaKMax, KmNai;
  real KmKo, KmpCa, ipCaMax, gCabMax, gNabMax;
  real v1RyR, Kfb, Krb, KSR, Nfb, Nrb, vmaxf, vmaxr, tautr, tauxfer;
  real gKpMax, Nao, Ko, Cao, Nai;

  real kaplus, kaminus, kbplus, kbminus, kcplus, kcminus, ncoop, mcoop;
  real fL, gL, omegaL, Pscale, 
       PCaL, PKL, iCahalf;
  real LTRPNtot, HTRPNtot, khtrpnplus, khtrpnminus, kltrpnplus;
  real kltrpnminus;
  real aL, bL, CMDNtot, KmCMDN, CSQNtot, KmCSQN;
  real EGTAtot, KmEGTA;

  real stim_start, stim_stop, stim_current, ist;

  void setConstants(); 
  void computeCurrents(const Ptv(real)& state, real t);

  void computeGateCoefs(const Ptv(real)& state);
  real totalCurrent();

  bool sdirk; // true if we use SDIRK solver.

  SDIRK* SDIRKsolver; //made specially for this class.

  //some coefficient functions:
  
  inline real beta_i(real Cai){
    real a1,a2;
    a1 = CMDNtot*KmCMDN/pow((Cai+KmCMDN),2.0);
    a2 = EGTAtot*KmEGTA/pow((Cai+KmEGTA),2.0);
    return 1.0/(1.0+a1+a2);
  }

  inline real beta_ss(real Cass){
    real a1,a2;
    a1 = CMDNtot*KmCMDN/pow((Cass+KmCMDN),2.0);
    a2 = EGTAtot*KmEGTA/pow((Cass+KmEGTA),2.0);
    return 1.0/(1.0+a1+a2); 
  }

  inline real beta_JSR(real CaJSR){
    return 1.0/(1.0+CSQNtot*KmCSQN/pow((CaJSR+KmCSQN),2.0));
  }

  inline real K_p(real V){
    return 1.0/(1.0+exp((7.488-V)/5.98));
  }	

public:
  Winslow();                // Default constructor
  virtual ~Winslow(){
    cout << "total number of iterations = " << solver->getWork() <<endl; 
  }

  virtual void evalStateDot(Vec(real)& ydot, const DynamicState& s);
  virtual real ionicCurrent(Vec(real)& state, real t); 
  virtual int evalSize() const {return 31;}
  virtual int getTransmembraneIndex() const {return 31;}
  virtual void setParameters(Ptv(real)& p );
  static void defineStatic (MenuSystem& menu, int level = MAIN);
  void define (MenuSystem& menu, int level = MAIN) 
     {defineStatic (menu,level);}

  virtual void scan   (MenuSystem& menu);

  virtual void innerLoop(Ptv(real)& /*auxx*/);

};

#define ClassType Winslow
#include <Handle.h>
#undef ClassType


#endif







