#include "Winslow_opt.h"

Winslow_opt::Winslow_opt():Winslow()
{

}



void Winslow_opt::recomputeJacobian(Mat(real)& J, const DynamicState& y)
{
  DPTRACE("Winslow::recomputeJacobian");
  real tmp1, deK, deKs, diKr, diKs, dito1, diK1, diKp, diCaK;
  real diNaCa, dipCa, diCab, dfb, drb, djupdCaNSR, djupdCai, dbetai;
  real dbetass, dbetaJSR, djTrpn; 
 
  /*The last two columns are computed with differences. They are computed first to
    ensure that all ionic currents etc. are updated*/
  tmp = y;
  tmp.y(31) += diff;
  //ydot2 = computeDerivative(tmp);
  evalStateDot(ydot2, tmp);  

  tmp = y;
  tmp.y(30) += diff;
  /*
  ydot3 = computeDerivative(tmp);  
  ydot1 = computeDerivative(y);*/
  evalStateDot(ydot3,tmp);
  evalStateDot(ydot1,y);


  for (int i = 1; i <= 31; i++){
    J(i,31) = (ydot2(i)-ydot1(i))*diffinv;
    J(i,30) = (ydot3(i)-ydot1(i))*diffinv;
  }

  //derivatives wrt Ki
  deK = -RTonF/Ki;
  deKs = -RTonF/(Ki+0.01833*Nai);
  diKr = -gKrMax*fKo*rV*xKr*deK;
  diKs = -gKsMax*xKs*xKs*deKs;
  dito1 = -gto1Max*xto1*yto1*deK;
  diK1 = -gK1Max*K1inf*(Ko/(Ko+KmK1))*deK;
  diKp = -gKpMax*K_p(V)*deK;
  diCaK = PKLprime/cSC*yL*open*1000*F*VFonRT*expVFonRT/(expVFonRT-1);

  //derivatives wrt Cai
  diNaCa = - kNaCa*(5000.0/(pow(KmNa,3.0)+pow(Nao,3.0)))*(1.0/(KmCa+Cao))
    *(1.0/(1.0+ksat*exp((eta-1)*V*FonRT)))*exp((eta-1)*VFonRT)*pow(Nao,3.0);
  dipCa = ipCaMax*KmpCa/((KmpCa+Cai)*(KmpCa+Cai));
  diCab = gCabMax*RTonF/(2*Cai);

  dfb = Nfb*pow(Cai,(Nfb-1.0))/pow(Kfb,Nfb);
  drb = Nrb*pow(CaJSR,(Nrb-1.0))/pow(Krb,Nrb);
  tmp1 = (1+fb+rb)*(1+fb+rb);
  djupdCaNSR = KSR*(-vmaxr-fb*(vmaxr+vmaxf))/tmp1*drb;
  djupdCai = KSR*(vmaxf*(1+rb)+vmaxr*rb)/tmp1*dfb; 


  J(1,1)   = -alpham-betam;
  J(2,2)   = -alphah-betah;
  J(3,3)   = -alphaj-betaj;
  J(4,4)   = -1.0/tauxKr; 
  J(5,5)   = -1.0/tauxKs;
  J(6,6)   = -alphaxto1-betaxto1;
  J(7,7)   = -alphayto1-betayto1;
  J(8,4)   = -gKrMax*fKo*rV*(V-eK)*aCap*cSC/(vMyo*1000.0*F);
  J(8,5)   = -gKsMax*2*xKs*(V-eK)*aCap*cSC/(vMyo*1000.0*F);
  J(8,6)   = -gto1Max*yto1*(V-eK)*aCap*cSC/(vMyo*1000.0*F);
  J(8,7)   = -gto1Max*xto1*(V-eK)*aCap*cSC/(vMyo*1000.0*F);
  J(8,8)   = -(diKr+diKs+dito1+diK1+diKp+diCaK)*aCap*cSC/(vMyo*1000.0*F);
  J(8,26)  = -PKLprime/cSC*yL*1000*F*VFonRT*(Ki*expVFonRT-Ko)/(expVFonRT-1)
                  *aCap*cSC/(vMyo*1000.0*F);
  J(8,27)  = -PKLprime/cSC*open
             *1000*F*VFonRT*(Ki*expVFonRT-Ko)/(expVFonRT-1)*aCap*cSC/(vMyo*1000.0*F);

  dbetai = (beta_i(Cai+diff)-betai)/diff;
  djTrpn = LTRPNtot* kltrpnplus*(1.0 - LTRPNCa)+HTRPNtot*khtrpnplus*(1.0 - HTRPNCa);
  J(9,9)   = dbetai*(jXfer-jUp-jTrpn-(iCab-2*iNaCa+ipCa)*
		     aCap*cSC/(2*vMyo*1000*F))+betai*(-1.0/tauxfer-djupdCai-
                     djTrpn-(diCab-2.0*diNaCa+dipCa)*aCap*cSC/(2*vMyo*1000*F));  
  J(9,10)  = -betai*djupdCaNSR;
  J(9,28)  = betai*(-kltrpnplus*Cai-kltrpnminus);
  J(9,29)  = betai*(-khtrpnplus*Cai-khtrpnminus);
  J(9,30)  = betai/tauxfer;
  J(10,10) = djupdCaNSR*vMyo/vNSR-1.0/tautr*vJSR/vNSR;
  J(10,11) = 1.0/tautr*vJSR/vNSR; 

  dbetaJSR = (beta_JSR(CaJSR+diff)-betaJSR)/diff;
  J(11,10) = betaJSR*1.0/tautr; 
  J(11,11) = dbetaJSR*(jTr-jRel)-betaJSR/tautr;
  J(11,13) = -betaJSR*v1RyR*(CaJSR-Cass);
  J(11,14) = J(11,13);

  J(12,12) = -kaplus*pow_n;

  J(13,12) = kaplus*pow_n;
  J(13,13) = -kaminus-kbplus*pow_m-kcplus;

  J(14,13) = kbplus*pow((Cass*1000.0),mcoop);
 
  alphaL      = 0.4*exp((V+2.0)/10.0);
  betaL       = 0.05*exp(-(V+2.0)/13.0);
  alphaprimeL = alphaL*aL;
  betaprimeL = betaL/bL;
  gammaL   =0.10375*Cass;
 
  J(16,16) = -4.0*alphaL-gammaL;
  J(16,17) = betaL;

  J(17,16) = 4.0*alphaL;
  J(17,17) = -betaL-3.0*alphaL-gammaL*aL;
  J(17,18) = 2.0*betaL;
 
  J(18,17) = 3.0*alphaL;
  J(18,18) = -2.0*betaL-2.0*alphaL-gammaL*aL*aL;
  J(18,19) = 3.0*betaL;

  J(19,18) = 2.0*alphaL;
  J(19,19) = -3.0*betaL-alphaL-gammaL*aL*aL*aL;
  J(19,20) = 4.0*betaL;

  J(20,19) = alphaL;
  J(20,20) = -4.0*betaL-fL-gammaL*pow(aL,4.0);
  J(20,26) = gL;

  J(21,16) = gammaL;
  J(21,21) = -4.0*alphaprimeL-omegaL;
  J(21,22) = betaprimeL;

  J(22,17) = gammaL*aL;
  J(22,21) = 4.0*alphaprimeL;
  J(22,22) = -betaprimeL-3.0*alphaprimeL-omegaL/bL;
  J(22,23) = 2.0*betaprimeL;

  J(23,18) = gammaL*aL*aL;
  J(23,22) = 3.0*alphaprimeL;
  J(23,23) = -2.0*betaprimeL-2.0*alphaprimeL-omegaL/(bL*bL);
  J(23,24) = 3.0*betaprimeL;

  J(24,19) = gammaL*aL*aL*aL;
  J(24,23) = 2.0*alphaprimeL;
  J(24,24) = -3.0*betaprimeL-alphaprimeL-omegaL/(bL*bL*bL);
  J(24,25) = 4.0*betaprimeL;

  J(25,20) = gammaL*pow(aL,4.0);
  J(25,24) = alphaprimeL;
  J(25,25) = -4.0*betaprimeL-omegaL/pow(bL,4.0);
 
  J(27,27) = -1.0/tauy;
  J(28,9)  = kltrpnplus*(1.0 - LTRPNCa)-kltrpnminus * LTRPNCa;		     
  J(28,28) = -kltrpnplus*Cai-kltrpnminus;
  J(29,9)  = khtrpnplus*(1.0 - HTRPNCa)-khtrpnminus * HTRPNCa;
  J(29,29) = -khtrpnplus*Cai-khtrpnminus;

  dbetass      = (beta_ss(Cass+diff)-betass)/diff;
  
  J(30,9)  = betass*1.0/tauxfer*vMyo/vSS;
  J(30,11) = betass*v1RyR*(po1+po2)*vJSR/vSS;
  J(30,13) = betass*v1RyR*(CaJSR-Cass)*vJSR/vSS;
  J(30,14) = J(30,13);
  J(30,26) = -betass*iCaMax*yL*aCap*cSC/(2*vSS*1000*F);
  J(30,27) = -betass*iCaMax*open*aCap*cSC/(2*vSS*1000*F);

  J(31,1)  = -gNaMax*3*m*m*h*j*(V-eNa);
  J(31,2)  = -gNaMax*m*m*m*j*(V-eNa); 
  J(31,3)  = -gNaMax*m*m*m*h*(V-eNa);
  J(31,4)  = -gKrMax*fKo*rV*(V-eK);
  J(31,5)  = -1.0/tauxKs;
  J(31,6)  = -gto1Max*yto1*(V-eK);
  J(31,7)  = -gto1Max*xto1*(V-eK);
  J(31,8)  = -(diKr+diKs+dito1+diK1+diKp+diCaK);
  J(31,9)  = -(diNaCa+dipCa+diCab);
  J(31,26) = -iCaMax*yL-PKLprime/cSC*yL
             *1000*F*VFonRT*(Ki*expVFonRT-Ko)/(expVFonRT-1);
  J(31,27) = -iCaMax*open-PKLprime/cSC*open
             *1000*F*VFonRT*(Ki*expVFonRT-Ko)/(expVFonRT-1); 

}

void Winslow_opt::setConstantJacobianElms(Mat(real)& J)
{
  DPTRACE("Winslow::setConstantJacobianElms");

  J.elm(12,13) = kaminus;
  J.elm(13,14) = kbminus;
  J.elm(13,15) = kcminus;
  J.elm(14,14) = -kbminus;
  J.elm(15,13) = kcplus;
  J.elm(15,15) = -kcminus;
  J.elm(16,21) = omegaL;
  J.elm(17,22) = omegaL/bL; 
  J.elm(18,23) = omegaL/(bL*bL);
  J.elm(19,24) = omegaL/(bL*bL*bL);
  J.elm(20,25) = omegaL/pow(bL,4.0);
  J.elm(26,20) = fL;
  J.elm(26,26) = -gL;
}

void Winslow_opt::forwBackLU( Mat(real)& A, Vec(real)& x, Vec(real) b)
{  
  real sum;
  /*
  if (x.size() != 31)
    cout << "Wrong size of solution vector..." << endl;
  if (b.size() != 31)
    cout << "Wrong size of RHS vector..." << endl;
  */
  //solve the system Ly=b  
  for (int i = 1; i <= 7; i++)
    y(i) = b(i);
  sum = 0;
  for (int j = 4; j <= 7; j++)
    sum += A(8,j)*y(j);
  y(8) = (b(8)-sum);
  y(9) = b(9);
  sum = A(10,9)*y(9);
  y(10) = (b(10)-sum);
  sum = A(11,10)*y(10);
  y(11) = (b(11)-sum);
  y(12) = b(12);
  sum = A(13,12)*y(12);
  y(13) = (b(13)-sum);
  sum = A(14,13)*y(13);
  y(14) = (b(14)-sum);
  sum = A(15,13)*y(13)+A(15,14)*y(14);
  y(15) = (b(15)-sum);
  y(16) = b(16);
  sum = A(17,16)*y(16);
  y(17) = (b(17)-sum);
  sum = A(18,17)*y(17);
  y(18) = (b(18)-sum);
  sum = A(19,18)*y(18);
  y(19) = (b(19)-sum);
  sum = A(20,19)*y(19);
  y(20) = (b(20)-sum);
  sum = 0;
  for (int j = 16; j <= 20; j++)
    sum += A(21,j)*y(j);
  y(21) = (b(21)-sum);
  sum = 0.0;
  for (int j = 17; j <= 21; j++)
    sum += A(22,j)*y(j);
  y(22) = (b(22)-sum);
  sum = 0.0;
  for (int j = 18; j <= 22; j++)
    sum += A(23,j)*y(j);
  y(23) = (b(23)-sum);
  sum = 0.0;
  for (int j = 19; j <= 23; j++)
    sum += A(24,j)*y(j);
  y(24) = (b(24)-sum);
  sum = 0.0;
  for (int j = 20; j <= 24; j++)
    sum += A(25,j)*y(j);
  y(25) = (b(25)-sum);
  sum = 0.0;
  for (int j = 20; j <= 25; j++)
    sum += A(26,j)*y(j);
  y(26) = (b(26)-sum);
  y(27) = b(27);
  sum = 0.0;
  for (int j = 9; j <= 11; j++)
    sum += A(28,j)*y(j)+A(28,j+4)*y(j+4);
  y(28) = (b(28)-sum);
  sum = 0.0;
  for (int j = 9; j <= 11; j++)
    sum += A(29,j)*y(j)+A(29,j+4)*y(j+4);
  sum += A(29,28)*y(28);
  y(29) = (b(29)-sum);
  sum = 0.0;
  for (int j = 9; j <= 11; j++)
    sum += A(30,j)*y(j)+A(30,j+4)*y(j+4);
  for (int j = 26; j <= 29; j++)
    sum += A(30,j)*y(j);
  y(30) = (b(30)-sum);
  sum = 0.0;
  for (int j = 1; j <= 15; j++)
    sum += A(31,j)*y(j);
  for (int j = 26; j <= 30; j++)
    sum += A(31,j)*y(j);
  y(31) = (b(31)-sum);

  //solve Ux=y
  x(31) = y(31)/A(31,31);
  sum = A(30,31)*x(31);
  x(30) = (y(30)-sum)/A(30,30);
  sum = A(29,30)*x(30)+A(29,31)*x(31);
  x(29) = (y(29)-sum)/A(29,29);
  sum = A(28,29)*x(29)+A(28,30)*x(30)+A(28,31)*x(31);
  x(28) = (y(28)-sum)/A(28,28);
  sum = A(27,31)*x(31);
  x(27) = (y(27)-sum)/A(27,27);
  sum  = A(26,30)*x(30)+A(26,31)*x(31);
  x(26) = (y(26)-sum)/A(26,26);
  sum = A(25,26)*x(26)+A(25,30)*x(30)+A(25,31)*x(31);
  x(25) = (y(25)-sum)/A(25,25);
  sum = A(24,25)*x(25)+A(24,26)*x(26)+A(24,30)*x(30)+A(24,31)*x(31);
  x(24) = (y(24)-sum)/A(24,24);
  sum = A(23,24)*x(24)+A(23,25)*x(25)+A(23,26)*x(26)
       +A(23,30)*x(30)+A(23,31)*x(31);
  x(23) = (y(23)-sum)/A(23,23);
  sum = 0.0;
  for (int j = 23; j <= 26; j++)
    sum += A(22,j)*x(j);
  sum += A(22,30)*x(30)+A(22,31)*x(31);
  x(22) = (y(22)-sum)/A(22,22);
  sum = 0.0;
  for (int j = 22; j <= 26; j++)
    sum += A(21,j)*x(j);
  sum += A(21,30)*x(30)+A(21,31)*x(31);
  x(21) = (y(21)-sum)/A(21,21);
  sum = 0.0;
  for (int j = 21; j <= 26; j++)
    sum += A(20,j)*x(j);
  sum += A(20,30)*x(30)+A(20,31)*x(31);
  x(20) = (y(20)-sum)/A(20,20);
  sum = 0.0;
  for (int j = 20; j <= 24; j++)
    sum += A(19,j)*x(j);
  sum += A(19,30)*x(30)+A(19,31)*x(31);
  x(19) = (y(19)-sum)/A(19,19);
  sum = A(18,19)*x(19)+A(18,21)*x(21)+A(18,22)*x(22)+A(18,23)*x(23)
    +A(18,30)*x(30)+A(18,31)*x(31);
  x(18) = (y(18)-sum)/A(18,18);
  sum = A(17,18)*x(18)+A(17,21)*x(21)+A(17,22)*x(22)
    +A(17,30)*x(30)+A(17,31)*x(31);
  x(17) = (y(17)-sum)/A(17,17);
  sum = A(16,17)*x(17)+A(16,21)*x(21)+A(16,30)*x(30)+A(16,31)*x(31);
  x(16) = (y(16)-sum)/A(16,16);
  sum = A(15,30)*x(30);
  x(15) = (y(15)-sum)/A(15,15);
  sum = A(14,15)*x(15)+A(14,30)*x(30);
  x(14) = (y(14)-sum)/A(14,14);
  sum = A(13,14)*x(14)+A(13,15)*x(15)+A(13,30)*x(30);
  x(13) = (y(13)-sum)/A(13,13);
  sum = A(12,13)*x(13)+A(12,30)*x(30);
  x(12) = (y(12)-sum)/A(12,12);
  sum = A(11,13)*x(13)+A(11,14)*x(14);
  for (int j = 28; j <= 31 ; j++)
    sum += A(11,j)*x(j);
  x(11) = (y(11)-sum)/A(11,11);
  sum = A(10,11)*x(11);
  for (int j = 28; j <= 31 ; j++)
    sum += A(10,j)*x(j);
  x(10) = (y(10)-sum)/A(10,10);
  sum = A(9,10)*x(10);
  for (int j = 28; j <= 31 ; j++)
    sum += A(9,j)*x(j);
  x(9) = (y(9)-sum)/A(9,9);
  sum = A(8,26)*x(26)+A(8,27)*x(27)+A(8,31)*x(31);
  x(8) = (y(8)-sum)/A(8,8);
  for (int i = 7; i >= 1; i--){
    sum = A(i,31)*x(31);
    x(i) = (y(i)-sum)/A(i,i);
  }
}









