
#include <CombinedDynSys.h>

INIT_CLASS_INFO(CombinedDynSys)


//-----------------------------------------------------------------------------
CombinedDynSys::CombinedDynSys (int max_no_of_models)
//-----------------------------------------------------------------------------
{
         sys_state.rebind(new CombinedDynamicState);
  comunicate_state.rebind(new CombinedDynamicState);
  problems.redim(max_no_of_models);
  problem_no = 0;

}



//-----------------------------------------------------------------------------
void CombinedDynSys::comunicateBindTo(CombinedDynamicState& s)
//-----------------------------------------------------------------------------
{
  // only put up comunication if necessary
  if (comunicate_state.getPtr() != &s)
    {
      int i;
      comunicate_state.rebind(s);
      // part problems
      for (i = 1; i <= problem_no; i++)
	{
	  problems(i)->comunicate_state.rebind(s.part(i));
	}
    }
}



//-----------------------------------------------------------------------------
void CombinedDynSys::init()
//-----------------------------------------------------------------------------
{

  VecSimple(int) part_s(problem_no),
                 part_d(problem_no);

  int i; 
  for (i=1; i <= problem_no; i++)
    {
      part_s(i) = problems(i)->evalSize();
      part_d(i) = problems(i)->evalNoDiscrite();

    }

  CombinedDynamicState& ss = CAST_REF(sys_state(), CombinedDynamicState);

  ss.redim(part_s, part_d);
  setIV(sys_state());

  // distribute to part problems
  for (i = 1; i <= problem_no; i++)
    {
      problems(i)->sys_state.rebind(ss.part(i));
    }



}


//-----------------------------------------------------------------------------
void CombinedDynSys::add(DynSys& problem)
//-----------------------------------------------------------------------------
{
  if (problem_no+1 > problems.size())
    errorFP("CombinedDynSys::add","max number of DynSys reached !");

  problem_no++;

  int s = problem.evalSize();
  int d = problem.evalNoDiscrite();
  problems(problem_no).rebind(problem);
  tots += s;
  totd += d;

}

//-----------------------------------------------------------------------------
void CombinedDynSys::setIV(DynamicState& s)
//-----------------------------------------------------------------------------
{
  CombinedDynamicState& ss = CAST_REF(s,CombinedDynamicState);
  for (int i = 1; i <= problem_no; i++)
    {
      problems(i)->setIV(ss.part(i));
    }

}



//-----------------------------------------------------------------------------
int CombinedDynSys::evalNoConstraints() const
//-----------------------------------------------------------------------------
{
  int tot = 0;
  int i;
  for (i=1; i <= problem_no; i++)
    tot += problems(i)->evalNoConstraints();

  return tot;

}


//-----------------------------------------------------------------------------
void CombinedDynSys::evalConstraints(Vec(real)& results, DynamicState& s)
//-----------------------------------------------------------------------------
{
  CombinedDynamicState& ss = CAST_REF(s, CombinedDynamicState);
  real* ydtptr = results.getPtr0();
  int c;
  for (int i=1; i<= problem_no; i++)
    {
      c = problems(i)->evalNoConstraints();
      ((VecSimplest(real)&)(temp)).redim(ydtptr,c);
      problems(i)->evalConstraints(temp, ss.part(i));
      ydtptr +=  c;

    }
}


//-----------------------------------------------------------------------------
void CombinedDynSys::handleRoot(DynamicState& s, int i)
//-----------------------------------------------------------------------------
{
  CombinedDynamicState& ss = CAST_REF(s, CombinedDynamicState);

  int k = 0;
  int j;

  for (j=1; j <= problem_no; j++)
    {
      if (i < k + problems(i)->evalNoConstraints() )
	{ problems(j)->handleRoot(ss.part(i),i-k); break; }
      else { k += problems(i)->evalNoConstraints(); }
    }
}
