#ifndef CombinedDynSys_h_IS_INCLUDED
#define CombinedDynSys_h_IS_INCLUDED

#include <VecSimplest_Handle_DynSys.h>
#include <CombinedDynamicState.h>

/*<CombinedDynSys:*/
class CombinedDynSys: public virtual DynSys
{
  Vec(real) temp;

  protected:

  VecSimplest(Handle(DynSys))           problems; // the part problems
  int                                 problem_no; // no of problems added
  int                                       tots; // total size of state
  int                                       totd; // total discrite state size


  void comunicateBindTo(CombinedDynamicState& s);

  public:

  CombinedDynSys (int max_no_of_models = 1);
  virtual ~CombinedDynSys () {}

  virtual void init();

  virtual void add(DynSys& problem);

  int evalSize() const
  { return tots; }

  int evalNoDiscrite() const
  { return totd; }

  virtual void setIV(DynamicState& s);

  virtual int evalNoConstraints() const;

  // evaluate the constraints on the system
  virtual void evalConstraints(Vec(real)& results, DynamicState& s);

  // invoke switch of model
  virtual void handleRoot(DynamicState& s, int i);


  COPY_CONSTRUCTOR(CombinedDynSys)
  ASSIGNMENT_OPERATOR(CombinedDynSys)
  VIRTUAL_CAST(CombinedDynSys)
  CLASS_INFO

};
/*>CombinedDynSys:*/

/*Class:CombinedDynSys
NAME: CombinedDynSys - specification of a combined dynamical system.

SYNTAX: @CombinedDynSys

KEYWORDS:
    base class, IVP , ODE

DESCRIPTION:
  The class is derived from the base class "DynSys" and is a part of
  the hirarchy of dynamical systems. The current class implements the
  interface of "DynSys" by a collection of sub systems. The sub systems
  are combined to a global dynamical system. The member function "add"
  is used to specify these sub systems.
  For communication between the sub-systems the "comState" member function
  of "DynSys" can be used. A member function should for a part problem
  therfore explicitly tell whether the member function "comState" apply.


CONSTRUCTORS AND INITIALIZATION:
  The constructor takes the maximim numbers of models that can be added.
  Further initialization is done by the add member function. Please note
  that the dynamical system specified by this class can not be solved by any
  of the ODE solvers, which actually require the derived class
  "CombinedExplicitODE" as input.


MEMBER FUNCTIONS:

  $\begin{description}

  `\item[`"add"`]` -
        adds a sub system to the global dynamical system.


   `\item[`"init"`]` -
         initializes the system, used by the solver to
	 initialize the data structure before any solution is
	 sought.

   `\item[`"evalSize"`]` -
         return the total size of the system as the sum
         of the sizes of the sub systems.

  `\item[`"evalNoDiscrite"`]` -
        returns the total number of discrite states as the sum of the
        discrite sizes of the sub systems.

   `\item[`"setIV"`]` -
        specifies the initial values by calling the sub systems "setIV"
        member function.

   `\item[`"evalNoConstraints"`]` -
         returns with the number of constraints as the sum of the sub
         systems number of constraints.

   `\item[`" evalConstraints"`]` -
         the constraint function is evaluated by calling the sub systems
         evalConstraints.

   `\item[`"handleRoot"`]` -
         the action to be taken when a root is encountered,
	 calls handleRoot to the sub systems having a root.


  $\end{description}


EXAMPLES:
  see Combined

SEE ALSO:

   class "CombinedExplicitODE", class "ExplicitODEeqUDC"

DEVELOPED BY:

   SINTEF Applied Mathematics, Oslo, Norway.

AUTHOR:

   Johan Berge

End: */




#endif
