#include <CombinedDynamicState.h>

INIT_CLASS_INFO(CombinedDynamicState)



//-------------------------------------------------------------------------
CombinedDynamicState::CombinedDynamicState(const CombinedDynamicState& state)
//-------------------------------------------------------------------------
{
  redim(state.part_size, state.part_disc);
  (*this) = state;
}


//-------------------------------------------------------------------------
void CombinedDynamicState::makeItSimilar(Handle(DynamicState)& s)
//-------------------------------------------------------------------------
{
  s.rebind(new CombinedDynamicState(*this));
}


//-------------------------------------------------------------------------
bool CombinedDynamicState::redim(const VecSimple(int)& parts,
				    const VecSimple(int)& partd)
//-------------------------------------------------------------------------
{

  if (parts.size() != partd.size())
    errorFP("CombinedDynamicState::redim",
	    "illigal length, the vectors must have same size !");

  part_states.redim(parts.size());

  int tots=0;
  int totd=0;
  int i;
  for (i=1; i<= parts.size(); i++)
    {
      tots += parts(i); totd += partd(i);
      part_states(i).rebind(new DynamicState);
    }

  bool retv;
  retv = DynamicState::redim(tots,totd);

  real* yptr    = (real*)y_.getPtr0();
  real* ydotptr = (real*)ydot_.getPtr0();
  real* dptr    = (real*)d_.getPtr0();

  for (i=1; i<= parts.size(); i++)
    {
      ((VecSimplest(real)&)part(i).y_).redim(yptr,parts(i));
      ((VecSimplest(real)&)part(i).ydot_).redim(ydotptr,parts(i));
      if (partd(i)>0)
	((VecSimplest(real)&)part(i).d_).redim(dptr,partd(i));
      yptr    += parts(i);
      ydotptr += parts(i);
      dptr    += partd(i);

      // set up pointer :
      part(i).t = this->t;
    }

  part_size.redim(parts.size());
  part_disc.redim(parts.size());

  part_size = parts;
  part_disc = partd;

  return retv;
}
