#ifndef CombinedDynamicState_h_IS_INCLUDED
#define CombinedDynamicState_h_IS_INCLUDED

#include <VecSimplest_Handle_DynamicState.h>

/*<CombinedDynamicState:*/
class CombinedDynamicState: public DynamicState
{

  VecSimplest(Handle(DynamicState)) part_states;
  VecSimple(int)                    part_size;
  VecSimple(int)                    part_disc;

  public:

  CombinedDynamicState() {}
  CombinedDynamicState(const CombinedDynamicState& s);

  DynamicState& part(int i) { return part_states(i)(); }

  CombinedDynamicState& operator=(const CombinedDynamicState& s)
  {
    DynamicState::operator=(s);
    return *this;
  }


  virtual bool redim(int states, int discrites = 0)
  {
    VecSimple(int) parts(1), partd(1);
    parts(1) = states;
    partd(1) = discrites;
    return redim (parts, partd);
  }


  bool redim(const VecSimple(int)& parts,
 	        const VecSimple(int)& partd);

  virtual void makeItSimilar(Handle(DynamicState)& s);

  CLASS_INFO
  VIRTUAL_CAST(CombinedDynamicState)
};
/*>CombinedDynamicState:*/

/*Class:CombinedDynamicState

NAME: CombinedDynamicState - Dynamic state, Combined systems

SYNTAX: @CombinedDynamicState

KEYWORDS: ODE, IVP, Dynamic State, Combined systems

DESCRIPTION:

  The class contains the state vector, state derivative, time and the discrite
  state vector inherited from the base class "DynamicState".
  In addition this class  partitions the 'global state vectors'
  into part states. These part states refers to the same memory as
  the global states. For combined dynamic systems represented by multiple
  sub systems the part states can be obtained by the member function "part".

CONSTRUCTORS AND INITIALIZATION:


MEMBER FUNCTIONS:
   $\begin{description}

    `\item[`"redim"`]` - redimensions the state vector and the discrite
       state vector. The argument vector length tells how many part states
       the class contains. Each entry of the vector is used to initialize
       the specific part.

       `\item[`"part"`]` - returns with the part state number given in
       the argument.

   $\end{description}

EXAMPLES:

SEE ALSO:

  class "DynamicState" and class "CombinedDynSys"

DEVELOPED BY:

   SINTEF Applied Mathematics, Oslo, Norway.

AUTHOR:

   Johan Berge

End: */


#define ClassType CombinedDynamicState
#include <Handle.h>
#undef ClassType


#endif
