#ifndef CombinedExplicitODE_h_IS_INCLUDED
#define CombinedExplicitODE_h_IS_INCLUDED

#include <CombinedDynSys.h>

/*<CombinedExplicitODE:*/
class CombinedExplicitODE: public CombinedDynSys,
			   public ExplicitODEeqUDC
{
  VecSimplest(Handle(ExplicitODEeqUDC)) ex_problems; // the part problems
  Vec(real) ytemp;

  public:

  CombinedExplicitODE (int no_of_models = 1);

  virtual void add(DynSys& problem);

  // define the system
  virtual void evalStateDot(Vec(real)& ydot, const DynamicState& s);


  COPY_CONSTRUCTOR(CombinedExplicitODE)
  ASSIGNMENT_OPERATOR(CombinedExplicitODE)
  VIRTUAL_CAST(CombinedExplicitODE)
  CLASS_INFO

};
/*>CombinedExplicitODE:*/


/*Class:CombinedExplicitODE
NAME: CombinedExplicitODE - specification of a combined dynamical system of
                            explicit ordinary differential equations.

SYNTAX: @CombinedExplicitODE

KEYWORDS:
    ODE , IVP, Combined Systems.

DESCRIPTION:
  The class is derived from the class "CombinedDynSys" for combined dynamical
  systems. This class requires that the system is specified as a system
  of explicit ordinary differenetial equations. The sub systems are specified
  by instances of a class derived from the class "ExplicitODEeqUDC".
  Please also consult the documantation of "CombinedDynSys" and
  "ExplicitODEeqUDC". Within the "evalStateDot" member function of the
  sub systems the argument state can be accessed through
  the "comState" member function for communication between the sub systems.


CONSTRUCTORS AND INITIALIZATION:
  The constructor takes the maximim numbers of models that can be added.

MEMBER FUNCTIONS:

  $\begin{description}

  \item[`"add"`]` -  adds a subsystem to the global dynamical system.

  \item[`"evalStateDot"`]` -  evaluates the derivative given the state
       in the argument. This is accomplished by calling the sub systems
       "evalStateDot" member function. Communication between models with
       the argument state can be achived through the
       "comState" member function of "DynSys".

  $\end{description}


EXAMPLES:
   See class "Combined"

SEE ALSO:

   class "ExplicitODEeqUDC", class "CombinedDynSys"

DEVELOPED BY:

   SINTEF Applied Mathematics, Oslo, Norway.

AUTHOR:

   Johan Berge

End: */



#endif
