#ifndef EmsODEsolver_h_IS_INCLUDED
#define EmsODEsolver_h_IS_INCLUDED

#include <ExplicitODEsolver.h>
#include <ErkODEsolver.h>

/*<AdamsMoulton4:*/
class AdamsMoulton4 : public ExplicitODEsolver
{
  protected:

  Vec(real) f0;
  Vec(real) f1;
  Vec(real) f2;
  Vec(real) f3;

  Vec(real) yp;  // predictor
  Vec(real) yc;  // corrector
  Handle(DynamicState) s0;

  int   nsteps; // internal counter used in the initial steps

  // startmethod is by defaullt set to RungeKuttaF45

  Handle(ExplicitODEsolver) startmethod;
  int ststeps;
  public:

   AdamsMoulton4() {}
   AdamsMoulton4(ODEsolver_prm& pm);
  ~AdamsMoulton4() {}

  virtual void newIntegrate();

  virtual void restartIntegrate()
  {
    ExplicitODEsolver:: restartIntegrate();
    nsteps = 3;
  }

  virtual void  retryNextStep()
  {
    ExplicitODEsolver::retryNextStep();
    nsteps = 3;
  }

  virtual int  getWork() const
  {
    return startmethod->getWork() + work;
  }

  void attachStartMethod(ExplicitODEsolver& method);

  virtual void advance  (real dt);
  virtual real advanceE (real dt);

  int getOrder() const {return 4;}
  int getNoOfStepsMethod() const { return 3; }


  COPY_CONSTRUCTOR(AdamsMoulton4)
  ASSIGNMENT_OPERATOR(AdamsMoulton4)
  CLASS_INFO
};
/*>AdamsMoulton4:*/

/*Class:AdamsMoulton4

NAME: AdamsMoulton4 - numerical solution of ordinary differential equations
      by the fourth order Adams-Moulton method.

SYNTAX: @AdamsMoulton4

KEYWORDS:  solver, O(`\(`h^4`\)`) , IVP

DESCRIPTION:
   The class implements the multi step method known as Adams-Moltoun
   method. The method requires three values of the past, so a start method
   which is single step have to be given to the class. Please note that
   every time the step size is changed the startmethod is invoked so
   if the step size changes frequently the startmethod will become
   the main solver.

CONSTRUCTORS AND INITIALIZATION:


MEMBER FUNCTIONS:
  $\begin{description}

     `\item[`"advance"`]` - integrate one time step.

      `\item[`"advanceE"`]` -
       integrate one time step and return with an estimate of the
       local error.


  $\end{description}

EXAMPLES:
   See class "Oscillator"

SEE ALSO:

   class "Euler1" and class "RungeKuttaF45"

DEVELOPED BY:

   SINTEF Applied Mathematics, Oslo, Norway.

AUTHOR:

   Johan Berge
End: */


#endif
