#include <ExplicitODEsolver.h>

INIT_CLASS_INFO(ExplicitODEsolver)


//---------------------------------------------------------------------------
void ExplicitODEsolver::attach(DynSys& problem_)
//---------------------------------------------------------------------------
{
  problem.rebind(CAST_REF(problem_,ExplicitODEeqUDC));
  ODEsolver::attach(problem_);
}


//---------------------------------------------------------------------------
void ExplicitODEsolver::newIntegrate()
//---------------------------------------------------------------------------
{
  ODEsolver::newIntegrate();
  work = 0;

  problemEvalStateDot(current_state());

  for (int i=1; i<=no_of_states; i++)
    last_state().ydot(i) = current_state().ydot(i);

  internal_state.redim(no_of_states,no_of_discrites);

}


//---------------------------------------------------------------------------
void ExplicitODEsolver::restartIntegrate()
//---------------------------------------------------------------------------
{
  // make sure the derivative is updated as the system might have changed
  // this must be done before ODEsolver::restartIntegrate since last_state
  // is put to current_state

  problemEvalStateDot(current_state());

  ODEsolver::restartIntegrate();

}


//----------------------------------------------------------------------------
void  ExplicitODEsolver::retryNextStep()
//----------------------------------------------------------------------------
{
  ODEsolver::retryNextStep();

  // Note that the First Same As Last property is
  // ok since current_state is put to last_state
  // and ydot is ok !
}



//----------------------------------------------------------------------------
real ExplicitODEsolver::advanceE(real dt)
//----------------------------------------------------------------------------
{
  advance(dt);

  locerror = (current_state->yvec());

  current_state() = last_state();
  internal_state  = last_state();

  real dt2 = 0.5*dt;

  // should be regarded as one step
  advance(dt2);
  advance(dt2);

  last_state() = internal_state;
  last_dt = dt;
  real scalefactor = 1.0/(1.0 - pow(2.0, real(getOrder())));
  int i;
  for (i=1; i <= no_of_states; i++)
    locerror(i) = scalefactor*(current_state->y(i) - locerror(i));

  return problem->evalErrorEstimate(locerror);

}




//----------------------------------------------------------------------------
void ExplicitODEsolver::interpolate(real teta, DynamicState& s,
				    bool update_ydot)
//----------------------------------------------------------------------------
{
  if (teta<=0) { s = last_state(); }
  else
  if (teta>=1) { s = current_state(); }
  else
  if (last_dt == 0) { s = current_state(); }
  else
    {

      // third order cubic polynomial interpolation
      // -------------------------------------------

      int i;
      real t;
      real d;

      t = last_state->time() + teta*last_dt;
      for (i = 1; i <= s.size(); i++)
	{
	  d = (current_state->y(i) - last_state->y(i))/last_dt;
	  s.y(i) = last_state->y(i) + last_dt * teta*
	    (last_state->ydot(i) + teta * (d - last_state->ydot(i))
	     + teta* (teta - 1.0) * (current_state->ydot(i) - 2.0 * d +
				     last_state->ydot(i)));
	}

      s.time() = t;

      for (i=1; i <= no_of_discrites; i++)
	s.d(i) = last_state->d(i);


      if (update_ydot)
	problemEvalStateDot(s);
    }

}
