#include <RootFinder.h>


INIT_CLASS_INFO(RootFinder)
INIT_CLASS_INFO(Bisection)
INIT_CLASS_INFO(RegulaFalsi)

//-------------------------------------------------------------------------
real Bisection::rootfind( ODEsolver& s,
			  int constraint_index )
//-------------------------------------------------------------------------
{
  itt = 0;
  DynSys& problem = s.getProblem();
  problem.state().makeItSimilar(scratch);

  results.redim(problem.evalNoConstraints());

  real constraint_value = 1;
  real ts = s.prevState().time();
  real dt = s.lastDt()/2.0;
  real tt=0;
  real delta_t = s.lastDt();
  real teta;

  while (fabs(constraint_value) > within && itt < maxitt)
    {
      tt = ts + dt;
      teta = (tt-s.prevState().time())/delta_t;

      s.interpolate(teta, scratch(),false);

      problem.evalConstraints(results,scratch());
      constraint_value = results(constraint_index);

      if (constraint_value < 0)
	{ tt = ts; }
      else ts = tt;

      dt = dt/2.0;
      itt++;

    }

  return tt;


}





//-------------------------------------------------------------------------
real RegulaFalsi::rootfind(ODEsolver& s, int constraint_index )
//-------------------------------------------------------------------------
{
  itt = 0;
  DynSys& problem = s.getProblem();

  problem.state().makeItSimilar(scratch);
  results.redim(problem.evalNoConstraints());

  real tx0 = s.prevState().time();
  real tx1 = s.currentState().time();

  scratch() = s.prevState();
  problem.evalConstraints(results,scratch());
  real fx0 = results(constraint_index);

  scratch() = s.currentState();
  problem.evalConstraints(results,scratch());
  real fx1 = results(constraint_index);

  real fx2 = 1;
  real tx2 = 0;
  real teta;

  real delta_t = s.lastDt();

  while (fabs(fx2) > within  && itt < maxitt)
    {
      tx2 =  tx0 - fx0*(tx0-tx1) / (fx0 - fx1);
      teta = (tx2-s.prevState().time())/delta_t;

      s.interpolate(teta, scratch(),false);

      problem.evalConstraints(results,scratch());
      fx2 = results(constraint_index);

      if (sign(fx2)  != sign(fx0))
	{ tx1 = tx2; fx1 = fx2; }
      else { tx0 = tx2; fx0 = fx2; }

      itt++;

    }

  return tx2;


}
